//Generates a mandelbrot set of the polynomial with complex critical points = crits[0], ..., crits[3] 
//or a movie if you set one or more of the crits to vary e.g. with xpos
//the current example starts with cz^2(1-z) and moves criticals (0, 2/3) to (2/3,2/3)
//Generates a polynomial up to degree 5 from n crits and arbitrary c values in Params0.
//scal scales or rotates the crits together as a group
//mult multiplys all the resulting coefficients by a factor and add adds a constant term
//The array crits[4] holds up to 4 critical points
//Set ncrits to 2, 3 or 4 and input 0's for any unused crits

//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o mando crits.c -lm && ./mando
//To run successive compiled versions type ./mando

#include <stdio.h>
#import <complex.h>

double pi=3.14159;
int frames = 11; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
int ncrits=2;
double _Complex scal,mult,add;
double _Complex crits[4]={0,0.6666667,0,0};
int cs[7]={1,1,1,1,1,1,0};

void WriteParamFile(int frameno, double param1, double param2);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0;
double xend = crits[1];
double xpos; 
double ypos = 0;
scal=1.0;//1.4*cexp(1.0*pi*I/3); //Here is where to change scal!!!
mult=-1.0; //multiplicative scaling of the polynomial
add=0.0; //additive factor for the constant coefficient
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<ncrits;k++)
	crits[k]=crits[k]*scal;

for(k=0;k<frames;k++)
	{
	xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos, (double)ypos);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;
double _Complex  coefs[5]={0,0,0,0,0};
crits[0]=param1;
switch (ncrits)
	{
		case 2:
		{
			coefs[4]=mult*crits[0]*crits[1];
			coefs[3]=mult*1.0/2*(-crits[0]-crits[1]);
			coefs[2]=mult*1.0/3;
			break;
		}
		case 3:
		{
			coefs[4]=-mult*crits[0]*crits[1]*crits[2];
			coefs[3]=mult*1.0/2*(crits[0]*crits[1]+crits[1]*crits[2]+crits[2]*crits[0]);
			coefs[2]=mult*1.0/3*(-crits[0]-crits[1]-crits[2]);
			coefs[1]=mult*1.0/4;
			break;
		}
		case 4:
		{
			coefs[4]=mult*crits[0]*crits[1]*crits[2]*crits[3];
			coefs[3]=mult*1.0/2*(-crits[0]*crits[1]*crits[2]-crits[1]*crits[2]*crits[3]-crits[2]*crits[3]*crits[0]-crits[3]*crits[0]*crits[1]);
			coefs[2]=mult*1.0/3*(crits[0]*crits[1]+crits[0]*crits[2]+crits[0]*crits[3]+crits[1]*crits[2]+crits[1]*crits[3]+crits[2]*crits[3]);
			coefs[1]=mult*1.0/4*(-crits[0]-crits[1]-crits[2]-crits[3]);
			coefs[0]=mult*1.0/5;	
			break;
		}
	}

double specs[37];
double mandel=1;
double scale=100;
double screenX=0;
double screenY=0;
double cx=0;
double cy=0;
double thisfn=22;
double escType=0;
double orbitTrapE=0;
double escapeOnly=1;
double colorScheme=0;
double escapeBoundExp=1.666667;
double attractorEscBd=-4;
double maxIterations=128;
double dolog=0;
double kx=2;
double ky=0;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap epsilon)
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 
specs[18]=creal(coefs[0]);
specs[19]=cimag(coefs[0]);
specs[20]=creal(coefs[1]);
specs[21]=cimag(coefs[1]);
specs[22]=creal(coefs[2]);
specs[23]=cimag(coefs[2]);
specs[24]=creal(coefs[3]);
specs[25]=cimag(coefs[3]);
specs[26]=creal(coefs[4]);
specs[27]=cimag(coefs[4]);
specs[28]=creal(add);
specs[29]=cimag(add);
for(j=0;j<7;j++)
	specs[30+j]=cs[j];

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


