//generates a movie of separation of Julia sets of z^3-nz+c n=0-1 as the period 3 bulb bifurcates
//following the small dark heart using DHViewer
//Provides a good example of first using DarkHeart to explore 21 points as the island Mandelbrot set migrates
//The C-script then interpolates fact=15 times as many points between, to make a 20*15+1=301 frame movie
//To run this file from terminal put it in a folder, cd to the folder and input
//
//gcc -o mando cubicPd3bulbJ2b.c -lm && ./mando
//To run successive compiled versions type ./mando

#include <stdio.h>

double pi=3.14159;
int pts = 21;
int fact = 15;
int frames; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
double xpts[21]={-0.577124,-0.595093,-0.609715,-0.623314,-0.632363,-0.637355,-0.644398,-0.652101,-0.659535,-0.664377,-0.667166,-0.669942,-0.670929,-0.672146,-0.671741,-0.672039,-0.671711,-0.669552,-0.666674,-0.663834,-0.660290};
double ypts[21]={0.502180,0.489067,0.476107,0.460897,0.448960,0.433273,0.410901,0.396242,0.380990,0.364380,0.347930,0.332772,0.317510,0.301543,0.286711,0.273248,0.258057,0.244810,0.232938,0.220444,0.2102456};


void WriteParamFile(int frameno, double param1, double param2, double param3);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
frames=fact*(pts-1)+1;
int filetyp = 3;
int j,k,f;
double xstt = 0;
double xend = 1;
double xpos,apos,m; 
double ypos = 0;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	apos=xstt+(xend-xstt)*k/(frames-1);
	f=(int)((k)/fact);
	m=1.0*(k-fact*f)/fact;
       	if (k<frames-1) 
		{
            	xpos=(1-m)*xpts[f]+m*xpts[f+1];
           	ypos=(1-m)*ypts[f]+m*ypts[f+1];
		}
        else
 		{
            	xpos=xpts[f];
           	ypos=ypts[f];
		}
	WriteParamFile(k, (double)xpos, (double)ypos, (double)apos);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2, double param3)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[37];
double mandel=0;
double scale=4;
double screenX=0;
double screenY=0;
double cx=param1;
double cy=param2;
double thisfn=3;
double escType=0;
double orbitTrapE=3;
double escapeOnly=0;
double colorScheme=1;
double escapeBoundExp=2.7;
double attractorEscBd=-4;
double maxIterations=256;
double dolog=0;
double kx=3;
double ky=param3;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap epsilon)
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 
specs[18]=1;
specs[20]=1;
specs[26]=0;
specs[28]=0;
specs[31]=1;
specs[35]=1;
specs[36]=0;

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}