//generates a movie of separation of Julia sets of z^3-nz+c n=0-1 as the period 3 bulb bifurcates
//following the small dark heart using DHViewer
//Provides a good example of first using DarkHeart to explore 21 points as the island Mandelbrot set migrates
//The C-script then interpolates fact=15 times as many points between, to make a 20*15+1=301 frame movie
//To run this file from terminal put it in a folder, cd to the folder and input
//
//gcc -o mando cubicPd3dend2.c -lm && ./mando
//To run successive compiled versions type ./mando

#include <stdio.h>

double pi=3.14159;
int pts = 21;
int fact = 15;
int frames; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
double xpts[21]={-0.623680293560028, -0.628894805908203, -0.630166232585907, -0.62916624546051, -0.626488506793976, -0.622567296028137, -0.617689371109009, -0.612139344215393, -0.606188118457794, -0.599975645542145, -0.593931913375854, -0.588271915912628, -0.582922756671906, -0.578162968158722, -0.573993384838104, -0.570193767547607, -0.566845834255219, -0.563900232315063, -0.561144411563873, -0.558362126350403, -0.55551278591156};
double ypts[21]={0.63826048374176, 0.623315453529358, 0.607658863067627, 0.591189324855804, 0.573477447032928, 0.554854571819305, 0.535051643848419, 0.514323949813843, 0.492524981498718, 0.469939559698105, 0.446571230888367, 0.422654330730438, 0.398534536361694, 0.374234110116959, 0.350218892097473, 0.326418071985245, 0.303244709968567, 0.281037628650665, 0.259612828493118, 0.239119082689285, 0.219651997089386};


void WriteParamFile(int frameno, double param1, double param2, double param3);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
frames=fact*(pts-1)+1;
int filetyp = 3;
int j,k,f;
double xstt = 0;
double xend = 1;
double xpos,apos,m; 
double ypos = 0;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	apos=xstt+(xend-xstt)*k/(frames-1);
	f=(int)((k)/fact);
	m=1.0*(k-fact*f)/fact;
       	if (k<frames-1) 
		{
            	xpos=(1-m)*xpts[f]+m*xpts[f+1];
           	ypos=(1-m)*ypts[f]+m*ypts[f+1];
		}
        else
 		{
            	xpos=xpts[f];
           	ypos=ypts[f];
		}
	WriteParamFile(k, (double)xpos, (double)ypos, (double)apos);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2, double param3)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[37];
double mandel=0;
double scale=4;
double screenX=0;
double screenY=0;
double cx=param1;
double cy=param2;
double thisfn=3;
double escType=0;
double orbitTrapE=3;
double escapeOnly=0;
double colorScheme=1;
double escapeBoundExp=2.7;
double attractorEscBd=-4;
double maxIterations=256;
double dolog=0;
double kx=3;
double ky=param3;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap epsilon)
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 
specs[18]=1;
specs[20]=1;
specs[26]=0;
specs[28]=0;
specs[31]=1;
specs[35]=1;
specs[36]=0;

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}