//generates a movie of separation of Julia sets of z^3-nz+c n=0-1 as the period 3 bulb bifurcates
//following the small dark heart using DHViewer
//Provides a good example of first using DarkHeart to explore 21 points as the island Mandelbrot set migrates
//The C-script then interpolates fact=15 times as many points between, to make a 20*15+1=301 frame movie
//To run this file from terminal put it in a folder, cd to the folder and input
//
//gcc -o mando cubicPd3dendM.c -lm && ./mando
//To run successive compiled versions type ./mando

#include <stdio.h>

double pi=3.14159;
int pts = 13;
int fact = 15;
int frames; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
double xpts[13]={-0.6887043118476868, -0.7041381597518921, -0.7143892049789429, -0.7224076390266418, -0.7286672592163086, -0.7336320877075195, -0.7373762130737305, -0.7401300072669983, -0.7419779300689697, -0.7429866790771484, -0.7432857155799866, -0.7427910566329956, -0.7416023015975952};
double ypts[13]={0.5224001407623291, 0.507415235042572, 0.4937693178653717, 0.4800750017166138, 0.4661302864551544, 0.452080249786377, 0.4380369484424591, 0.4238862097263336, 0.409810334444046, 0.3958600163459778, 0.3820761442184448, 0.3684308230876923, 0.355006992816925};


void WriteParamFile(int frameno, double param1, double param2, double param3);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
frames=fact*(pts-1)+1;
int filetyp = 3;
int j,k,f;
double xstt = 0;
double xend = 0.6;
double xpos,apos,m; 
double ypos = 0;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	apos=xstt+(xend-xstt)*k/(frames-1);
	f=(int)((k)/fact);
	m=1.0*(k-fact*f)/fact;
       	if (k<frames-1) 
		{
            	xpos=(1-m)*xpts[f]+m*xpts[f+1];
           	ypos=(1-m)*ypts[f]+m*ypts[f+1];
		}
        else
 		{
            	xpos=xpts[f];
           	ypos=ypts[f];
		}
	WriteParamFile(k, (double)xpos, (double)ypos, (double)apos);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2, double param3)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[37];
double mandel=1;
double scale=0.04;
double screenX=param1;
double screenY=param2;
double cx=0;
double cy=0;
double thisfn=3;
double escType=0;
double orbitTrapE=2;
double escapeOnly=0;
double colorScheme=1;
double escapeBoundExp=2.7;
double attractorEscBd=-4;
double maxIterations=1024;
double dolog=0;
double kx=3;
double ky=param3;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap epsilon)
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 
specs[18]=1;
specs[20]=1;
specs[26]=0;
specs[28]=0;
specs[31]=1;
specs[35]=1;
specs[36]=0;

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}