//Generates a Mandelbrot set of the rational function with coefficients = (1/c)*(coefs[2]*z+coefs[3]*c+coefs[4]+coefs[5]/z)

//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o mando milnor.c -lm && ./mando
//To run successive compiled versions type ./mando

#include <stdio.h>
#import <complex.h>

double pi=3.14159;
int frames = 1; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
int ncrits=4;
double _Complex scal=1;
//double _Complex coefs[6]={0,0,1,-1,2,1};//Logistic
//int cs[7]={0,0,-1,0,-1,-1,0};
//double _Complex coefs[6]={0,0,1,-1,-2,1};//Snowshoe
//int cs[7]={0,0,-1,0,-1,-1,0};
//double _Complex coefs[6]={0,0,1,-1+0.4*I,2,1};//Spyglass
//int cs[7]={0,0,-1,0,-1,-1,0};
//double _Complex coefs[6]={0,0,1,-1,1.7+0.3*I,1};//Wink
//int cs[7]={0,0,-1,0,-1,-1,0};
//double _Complex coefs[6]={0,0,-1,-1,-50*(1+4*I),-1}; //Tie and Cane
//int cs[7]={0,0,-1,0,-1,-1,0};
//also set screenX=-50; screenY=-200;
//double _Complex coefs[6]={0,0,1,-1,3,1}; //Napoleon
//int cs[7]={0,0,-1,0,-1,-1,0};
//double _Complex coefs[6]={0,0,1,0.3-0.4*I,1.2+0.3*I,1}; //Sperm
//int cs[7]={0,0,-1,0,-1,-1,0};
double _Complex coefs[6]={0,0,1.0/(0.3-0.4*I),1.0/(0.3-0.4*I),-(1.2+0.3*I)/(0.3-0.4*I)/(0.3-0.4*I),1.0/(0.3-0.4*I)}; //Ring (Sperm a<>b)
int cs[7]={0,0,-1,0,-1,-1,0};
//double _Complex coefs[6]={0,0,1,0,0,1}; //Blood Clot
//int cs[7]={0,0,-1,0,0,-1,0};
//double _Complex coefs[6]={0,0,1,0,0,1}; //Alien (Blood Clot inverse)
//int cs[7]={0,0,1,0,0,1,0};
//double _Complex coefs[6]={0,0,1,0,2,1}; //Scallops
//int cs[7]={0,0,1,0,1,1,0};

void WriteParamFile(int frameno, double param1, double param2);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0;
double xend = 2;
double xpos; 
double ypos = 0;
scal=1.0;//1.4*cexp(1.0*pi*I/3); //Here is where to change scal!!!
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;

for(k=0;k<frames;k++)
	{
	xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos, (double)ypos);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

//coefs[1]=xpos; enter a varying substitution here to parametric;

double specs[37];
double mandel=1;
double scale=10;
double screenX=0;
double screenY=0;
double cx=0;
double cy=0;
double thisfn=28;
double escType=0;
double orbitTrapE=0;
double escapeOnly=0;
double colorScheme=4;
double escapeBoundExp=4;
double attractorEscBd=-4;
double maxIterations=1024;
double dolog=0;
double kx=2;
double ky=0;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap epsilon)
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 
specs[18]=creal(coefs[0]);
specs[19]=cimag(coefs[0]);
specs[20]=creal(coefs[1]);
specs[21]=cimag(coefs[1]);
specs[22]=creal(coefs[2]);
specs[23]=cimag(coefs[2]);
specs[24]=creal(coefs[3]);
specs[25]=cimag(coefs[3]);
specs[26]=creal(coefs[4]);
specs[27]=cimag(coefs[4]);
specs[28]=creal(coefs[5]);
specs[29]=cimag(coefs[5]);

for(j=0;j<7;j++)
	specs[30+j]=cs[j];

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


