//generates a movie of separation of Julia sets of z^3-nz+c as the period 3 bulb bifurcates
//following the small dark heart using DHViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//
//gcc -o mando z^3-z+c_MarianaJ.c -lm && ./mando
//To run successive compiled versions type ./mando

#include <stdio.h>

double pi=3.14159;
int frames = 201; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
double xpts[3]={-0.439277,-0.212117,-0.285536};
double ypts[3]={0.383797,0.2035868,0.3312125};

void WriteParamFile(int frameno, double param1, double param2);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xpos; 
double ypos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
for(k=0;k<frames;k++)
	{
	if (k<(frames-1)/2)
	{
		xpos=xpts[0]+(xpts[1]-xpts[0])*2*k/(frames-1);
		ypos=ypts[0]+(ypts[1]-ypts[0])*2*k/(frames-1);
	}
		else
	{
		xpos=xpts[1]+(xpts[2]-xpts[1])*2*(k-(frames-1)/2)/(frames-1);
		ypos=ypts[1]+(ypts[2]-ypts[1])*2*(k-(frames-1)/2)/(frames-1);
	}
	WriteParamFile(k, (double)xpos, (double)ypos);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[37];
double mandel=0;
double scale=3.5;
double screenX=0;
double screenY=0;
double cx=param1;
double cy=param2;
double thisfn=3;
double escType=0;
double orbitTrapE=0;
double escapeOnly=1;
double colorScheme=4;
double escapeBoundExp=1.666667;
double attractorEscBd=-4;
double maxIterations=512;
double dolog=0;
double kx=3;
double ky=1;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap epsilon)
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 
specs[18]=1;
specs[20]=1;
specs[26]=0;
specs[28]=0;
specs[31]=1;
specs[35]=1;
specs[36]=0;

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}