//save as djulia.c
//generates a movie of a Julia journey just outside the edge of the Mandelbrot set of z^d+c 
//main basin using DHViewer. Set variable d to give the degree k.
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o djuliao z^k+c_Julia.c -lm && ./djuliao
//To run successive compiled versions type ./djuliao

#include <stdio.h>
#include <math.h>
#import <complex.h>
double pi=3.14159;
int frames = 200; //number of movie frames - normally use 500-1000!
const char base[] = "Params";
double d=6; //degree d of x^d+c

void WriteParamFile(int frameno, double param1, double param2, double param3);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double astt = 0; //angle position
double aend = 2*(d-1)*pi;
double apos,xpos,ypos; 
double _Complex zpos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
for(k=0;k<frames;k++)
	{
	apos=astt+(aend-astt)*k/(frames-1);
	zpos=1.01*pow(d,-d/(d-1))*(d*cexp(I*apos/(d-1))-cexp(d*I*apos/(d-1)));
	xpos=creal(zpos);
	ypos=cimag(zpos);
	WriteParamFile(k, xpos, ypos, (double)d);
	}
return 0;
}

void WriteParamFile(int frameno, double param1, double param2, double param3)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[37];
double mandel=0;
double scale=3.5;
double screenX=0;
double screenY=0;
double cx=param1;
double cy=param2;
double thisfn=1;
double escType=0;
double orbitTrapE=0;
double escapeOnly=0;
double colorScheme=4;
double escapeBoundExp=1.666667;
double attractorEscBd=-4;
double maxIterations=1024;
double kx=param3;
double ky=0;
double oldfn=thisfn;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=thisfn;
specs[7]=escType; // abs, real, or imag
specs[8]=orbitTrapE; //orbit trap and dF
specs[9]=escapeOnly;
specs[10]=colorScheme;
specs[11]=escapeBoundExp; //escape bound exponent
specs[12]=attractorEscBd; //attractor escape bound 10^-k
specs[13]=maxIterations; //iterations
specs[14]=kx; //real k
specs[15]=ky; //imag k
specs[16]=oldfn; 
specs[17]=987.654321; 

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<37;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}
