//Generates C: y^2+(x^3+x^2+x+1)y=x^7+2*x^6+2*x^5+x^4 (genus 3, hyperelliptic) 
//to be processed with RZViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo Genus3.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <stdio.h>
#include<complex.h>
const int terms = 300;
const double pi=3.1415926535;
const int frames = 1; //number of movie frames - use n+1 for circular
const int spcno = 29+2+2*300+10;
const char base[] = "Genus3";
int ellm[300]={1,-1,-2,-1,-1,2,-2,1,4,1,-3,2,0,2,2,-1,8,-4,-3,1,4,3,4,-2,-3,0,-4,2,2,-2,-4,1,6,-8,2,-4,3,3,0,-1,-8,-4,-4,3,-4,-4,-5,2,1,3,-16,0,13,4,3,-2,6,-2,7,-2,0,4,-8,-1,0,-6,6,-8,-8,-2,-7,4,-9,-3,6,3,6,0,-10,1,0,8,-2,-4,-8,4,-4,-3,5,4,0,-4,8,5,3,-2,2,-1,-12,3,4,16,11,0,-4,-13,-9,4,2,-3,-6,2,7,-6,-4,-2,0,-7,-16,2,-14,0,16,4,3,8,13,9,8,0,8,-6,6,-6,4,8,-8,8,15,-2,10,7,0,-4,-2,9,-2,-3,4,-6,2,-3,32,-6,4,0,-20,10,-26,-1,-8,0,10,8,-6,2,14,4,-4,8,-12,4,-2,4,6,3,-14,-5,5,4,-9,0,0,4,-3,-8,-24,5,8,-3,6,2,7,-2,0,-1,-13,12,-9,-3,-12,-4,-4,16,8,-11,16,0,9,4,-11,-13,14,9,4,-4,8,-2,18,-3,0,6,-5,-2,-12,-7,13,-6,0,4,-12,2,17,0,5,-7,20,16,26,-2,-6,14,-2,0,-1,-16,0,-4,4,-3,-1,8,-12,-13,16,-1,16,-8,-6,0,8,-8,13,6,-13,-6,-10,-6,7,-4,-2,-8,0,8,9,8, 8,-15,-16,2,-10,-10,16,7,-6,0,16,4,24,2,-4,9,-17,2,-7,3,12,-4,0,-6
};

void WriteParamFile(int frameno, double param);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0.25;
double xend = 1.25;
double xpos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}
return 0;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[spcno];

double mandel=2;
double scale=80;
double screenX=0;
double screenY=0;
double cx=0;
double cy=0;
double altgamma=0;
double thisfn=4;
double split=2;
double orbitTrapE=0;
double addc=1;
double escapeonly=0;
double colsch=0;
double zetaterms=145;
double escapeboundExp=1.7;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=0;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=0;
double xm=0;
double xn=2;
double flip=0;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=zetaterms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip;

specs[29]=terms;
specs[30]=-997;

for (j=0;j<terms;j++)
	{
    specs[31+2*j]=ellm[j];
	specs[32+2*j]=0;
	}
specs[31+2*terms]=6;   //d no. gamma terms
specs[31+2*terms+1]=0; //gamma[0]
specs[31+2*terms+2]=0; //gamma[0]
specs[31+2*terms+3]=0; //gamma[0]
specs[31+2*terms+4]=1; //gamma[0]
specs[31+2*terms+5]=1; //gamma[1]
specs[31+2*terms+6]=1; //gamma[1]
specs[31+2*terms+7]=13555; //N=conductor
specs[31+2*terms+8]=1; //epsilon (sign)
specs[31+2*terms+9]=2;  //weight

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


