//Generates  elliptic curve of Maass cusp forms
//to be processed with RZViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo symm1.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <stdio.h>
#include<complex.h>
#include<math.h>
const int terms = 56;
const double pi=3.1415926535;
const int frames = 1; //number of movie frames - use n+1 for circular
const int spcno = 29+2+2*56+2*6;
const char base[] = "symm1";
double a=0.5; double b=1.5;
double an[]={1.0,-1.41421356237,-0.57735026919,1.0,0.4472135955,0.816496580928,-0.755928946018,0.0,-0.666666666667,-0.632455532034,0.301511344578,-0.57735026919,1.10940039245,1.06904496765,-0.258198889747,-1.0,-0.485071250073,0.942809041582,0.0,0.4472135955,0.436435780472,-0.426401432711,-0.208514414057,0.0,-0.8,-1.56892908111,0.962250448649,-0.755928946018,0.0,0.36514837167,1.25723711419,1.41421356237,-0.174077655956,0.68599434057,-0.338061701891,-0.666666666667,0.493196961916,0.0,-0.64051261522,0.0,-1.24939009511,-0.617213399848,-0.914991421996,0.301511344578,-0.298142397,0.29488391231,1.16691993198,0.57735026919,-0.428571428571,1.1313708499,0.280056016806,1.10940039245,-0.824163383692,-1.36082763488,0.134839972493};

void WriteParamFile(int frameno, double param);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0.25;
double xend = 1.25;
double xpos;
double _Complex a,b,c,ex,kk,tau=0;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}
return 0;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[spcno];

double mandel=2;
double scale=80;
double screenX=0;
double screenY=0;
double cx=0;
double cy=0;
double altgamma=0;
double thisfn=4;
double split=1;
double orbitTrapE=0;
double addc=1;
double escapeonly=0;
double colsch=0;
double zetaterms=terms;
double escapeboundExp=1.7;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=0;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=0;
double xm=0;
double xn=2;
double flip=0;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=zetaterms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip;

specs[29]=terms;
specs[30]=-995;

for (j=0;j<terms;j++)
	{
    specs[31+2*j]=creal(an[j]);
	specs[32+2*j]=cimag(an[j]);
	}
specs[31+2*terms]=2;   //d no. gamma terms
specs[31+2*terms+1]=0;
specs[31+2*terms+2]=0.5; //gamma[alpha]
specs[31+2*terms+3]=0;
specs[31+2*terms+4]=1.5; //gamma[beta]
specs[31+2*terms+5]=0;
specs[31+2*terms+6]=11; //q=conductor
specs[31+2*terms+7]=0;
specs[31+2*terms+8]=1; //epsilon
//specs[31+2*terms+8]=cos(2*pi*0.0)*pow(-1,e+1); //epsilon
specs[31+2*terms+9]=0;
//specs[31+2*terms+9]=sin(2*pi*0.0)*pow(-1,e+1); //epsilon
specs[31+2*terms+10]=1;  //weight
specs[31+2*terms+11]=0;  //twist pi powers in gamma terms as Dirichlet L-functions

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


