//Generates  elliptic curve L-function elleq  = E = EllipticCurve 17a [1,-1,1,-1,-14]  y^2 + x*y + y = x^3 - x^2 - x - 14
//to be processed with RZViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo 17a.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <stdio.h>
#include<complex.h>
const int terms = 590;
const double pi=3.1415926535;
const int frames = 1; //number of movie frames - use n+1 for circular
const int spcno = 29+2+2*590+6;
const char base[] = "17a";
int ellm[600]={1,-1,0,-1,-2,0,4,3,-3,2,0,0,-2,-4,0,-1,1,3,-4,2,0,0,4,0,-1,2,0,-4,6,0,4,-5,0,-1,-8,3,-2,4,0,-6,-6,0,4,0,6,-4,0,0,9,1,0,2,6,0,0,12,0,-6,-12,0,-10,-4,-12,7,4,0,4,-1,0,8,-4,-9,-6,2,0,4,0,0,12,2,9,6,-4,0,-2,-4,0,0,10,-6,-8,-4,0,0,8,0,2,-9,0,1,-10,0,8,-6,0,-6,8,0,6,0,0,-4,-14,0,-8,-6,6,12,4,0,-11,10,0,-4,12,12,8,3,0,-4,16,0,-16,-4,0,3,-6,0,-8,8,0,4,0,3,-12,6,0,2,-10,0,-16,-12,-3,0,-8,0,-2,-12,0,10,16,-9,24,6,0,4,-4,0,-9,2,12,-4,22,0,-4,0,0,-10,12,-6,-2,8,0,12,4,0,0,0,0,-8,-16,0,2,-2,0,-9,-18,0,-20,-3,0,10,24,0,12,-8,-12,2,0,0,8,-6,0,-8,-8,0,16,-6,0,0,-2,0,24,-20,3,14,-24,0,6,8,0,18,-6,-6,0,12,0,-4,-16,0,18,11,0,10,-18,0,8,12,0,-12,12,12,0,-8,0,-17,18,0,-8,-4,-18,-16,-16,0,-12,16,0,-4,22,0,-16,-1,0,6,0,0,14,8,-12,-24,-6,0,-16,4,0,0,-24,15,1,12,0,6,6,0,24,-6,0,10,-8,0,16,16,0,4,20,3,-12,0,0,8,28,0,-22,2,24,-12,-10,0,0,-14,0,-16,-4,-9,2,-24,0,-18,0,0,4,4,6,4,-8,0,-14,9,0,2,0,-12,8,12,0,-22,32,0,-18,4,0,0,-30,0,8,-10,0,-12,0,18,-3,2,0,8,12,0,28,-4,18,-4,24,0,6,0,0,0,-12,0,-8,-8,0,16,-24,0,0,-2,-12,-2,6,0,4,27,0,18,-24,0,6,20,0,1,-14,0,-8,10,-18,-24,0,0,26,-12,0,-8,-48,12,8,10,0,0,8,0,22,-8,0,18,-1,0,-40,-8,0,8,12,0,2,-16,0,-6,-16,0,-20,0,-27,2,28,0,-20,-24,0,28,34,-3,0,14,0,24,16,0,-6,-6,0,8,-2,0,32,-6,0,6,12,-6,16,0,0,-36,0,0,4,-4,-18,16,36,0,4,-18,0,11,-4,0,20,-30,0,18,20,0,6,-8,0,-4,-16,0,-40,-12,0,-12,-12,-36,20,0,0,-8,-2,0,-24,11,0,-18,-16,0,0,8,0,12,26,18,-36,-16,0,16,4,0,-7,12,36,16,12,0,-16,12,0,-22,0,0,6,16,0,-5,-12,0,-32,6,30,0,-24,0,48,-14,0,8,30,12,-8,8,0,6,-4,0,28,16,36,-12,-38,0,-32,0,0,24,-4,-21,-14,-1,0,12,-16,0,0,-18,-12,-6,4,0,-16,-24,0,2,18,0,-8,10,0,8,-24,0};

void WriteParamFile(int frameno, double param);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0.25;
double xend = 1.25;
double xpos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}
return 0;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[spcno];

double mandel=2;
double scale=80;
double screenX=0;
double screenY=0;
double cx=0;
double cy=0;
double altgamma=0;
double thisfn=4;
double split=2;
double orbitTrapE=0;
double addc=1;
double escapeonly=0;
double colsch=0;
double zetaterms=100;
double escapeboundExp=1.7;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=0;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=4;
double xm=0;
double xn=2;
double flip=0;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=zetaterms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip;

specs[29]=terms;
specs[30]=-997;

for (j=0;j<terms;j++)
	{
    specs[31+2*j]=ellm[j];
	specs[32+2*j]=0;
	}
specs[31+2*terms]=2;   //d no. gamma terms
specs[31+2*terms+1]=0; //gamma[0]
specs[31+2*terms+2]=1; //gamma[1]
specs[31+2*terms+3]=17; //N=conductor
specs[31+2*terms+4]=1; //epsilon (sign)
specs[31+2*terms+5]=2;  //weight

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


