//Generates  elliptic curve L-function elleq  = E = EllipticCurve 20a [0,1,0,4,4] y^2 = x^3 + x^2 + 4*x + 4
//to be processed with RZViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo 20a.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <stdio.h>
#include<complex.h>
const int terms = 590;
const double pi=3.1415926535;
const int frames = 1; //number of movie frames - use n+1 for circular
const int spcno = 29+2+2*590+6;
const char base[] = "20a";
int ellm[600]={1,0,-2,0,-1,0,2,0,1,0,0,0,2,0,2,0,-6,0,-4,0,-4,0,6,0,1,0,4,0,6,0,-4,0,0,0,-2,0,2,0,-4,0,6,0,-10,0,-1,0,-6,0,-3,0,12,0,-6,0,0,0,8,0,12,0,2,0,2,0,-2,0,2,0,-12,0,-12,0,2,0,-2,0,0,0,8,0,-11,0,6,0,6,0,-12,0,-6,0,4,0,8,0,4,0,2,0,0,0,6,0,14,0,4,0,-6,0,2,0,-4,0,-6,0,-6,0,2,0,-12,0,-11,0,-12,0,-1,0,2,0,20,0,0,0,-8,0,-4,0,18,0,-4,0,12,0,0,0,-6,0,6,0,-6,0,20,0,-6,0,4,0,-22,0,12,0,12,0,-10,0,0,0,18,0,-9,0,-4,0,-6,0,2,0,-24,0,-12,0,-10,0,-4,0,-2,0,0,0,8,0,-12,0,26,0,4,0,18,0,8,0,-4,0,12,0,-6,0,6,0,0,0,-16,0,24,0,10,0,-8,0,-4,0,-12,0,-10,0,1,0,-6,0,14,0,0,0,-6,0,6,0,-16,0,-24,0,14,0,10,0,3,0,-8,0,-12,0,0,0,0,0,-12,0,-6,0,4,0,6,0,-18,0,6,0,12,0,18,0,20,0,-8,0,0,0,26,0,-4,0,6,0,14,0,-8,0,12,0,19,0,-4,0,-30,0,-12,0,0,0,12,0,-20,0,-12,0,-2,0,2,0,-28,0,12,0,-22,0,-2,0,-6,0,0,0,12,0,24,0,2,0,-4,0,-12,0,8,0,2,0,-2,0,2,0,12,0,0,0,-20,0,12,0,-30,0,-10,0,8,0,18,0,12,0,24,0,24,0,-3,0,22,0,-2,0,-22,0,6,0,-12,0,26,0,2,0,12,0,-28,0,-4,0,6,0,0,0,-10,0,-6,0,-36,0,0,0,-8,0,2,0,16,0,-30,0,-8,0,11,0,0,0,-34,0,-36,0,24,0,-6,0,8,0,36,0,26,0,-6,0,-6,0,4,0,0,0,36,0,2,0,12,0,-24,0,8,0,-3,0,6,0,6,0,12,0,6,0,0,0,-40,0,-4,0,26,0,-24,0,30,0,14,0,-8,0,-30,0,4,0,44,0,0,0,-4,0,-6,0,-24,0,4,0,-24,0,-2,0,26,0,20,0,0,0,-36,0,0,0,-24,0,-4,0,-36,0,-18,0,-6,0,18,0,6,0,4,0,-16,0,-14,0,0,0,12,0,-6,0,14,0,-4,0,24,0,13,0,12,0,12,0,6,0,24,0,0,0,14,0,20,0,-2,0,26,0,2,0,-24,0,16,0,4,0,-30,0,-20,0,0,0,-18,0,6,0,-22,0,30,0,8,0,24,0,6,0,-22,0,-52,0,12,0,0,0,-2,0,-6,0,16,0,-36,0,18,0,12,0,-16,0,0,0};

void WriteParamFile(int frameno, double param);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0.25;
double xend = 1.25;
double xpos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}
return 0;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[spcno];

double mandel=2;
double scale=80;
double screenX=0;
double screenY=0;
double cx=0;
double cy=0;
double altgamma=0;
double thisfn=4;
double split=2;
double orbitTrapE=0;
double addc=1;
double escapeonly=0;
double colsch=0;
double zetaterms=100;
double escapeboundExp=1.7;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=0;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=4;
double xm=0;
double xn=2;
double flip=0;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=zetaterms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip;

specs[29]=terms;
specs[30]=-997;

for (j=0;j<terms;j++)
	{
    specs[31+2*j]=ellm[j];
	specs[32+2*j]=0;
	}
specs[31+2*terms]=2;   //d no. gamma terms
specs[31+2*terms+1]=0; //gamma[0]
specs[31+2*terms+2]=1; //gamma[1]
specs[31+2*terms+3]=20; //N=conductor
specs[31+2*terms+4]=1; //epsilon (sign)
specs[31+2*terms+5]=2;  //weight

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


