//Generates a movie parameter file set of zeta multiplicative Mandelbrot sets from c=-20 to c=2 using RZViewer
//to be run with RZViewer
//To run this file from Mac Terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo rzjulia.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <stdio.h>
#include<complex.h>

const double pi=3.1415926535;
const int frames = 201; //number of movie frames - normally use 500-1000!
const int spcno = 29;
const char base[] = "params";

void WriteParamFile(int frameno, double param);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt =-20;
double xend = 20;
double xpos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}
return 0;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[spcno];
double mandel=1;
double scale=80;
double screenX=0;
double screenY=0;
double cx=param;
double cy=0;
double altgamma=0;
double thisfn=1;
double split=1;
double orbitTrapE=2;
double addc=0;
double escapeonly=0;
double zetaterms=100;
double colsch=1;
double escapeboundExp=1.666667;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=1;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=0;
double xm=1;
double xn=1;
double flip=0;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=zetaterms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip; 
 
sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}