//Generates a movie parameter file set of alternating zetaF = prod[1+n^(-z)]^-1
//to be processed with RZViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo factoriz5.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <stdio.h>
#include<complex.h>
const int terms = 250;
const double pi=3.1415926535;
const int frames = 1; //number of movie frames - use n+1 for circular
const int spcno = 29+2+2*250;
const char base[] = "Factor0";
int factriz[250]={1,-1,-1,0,-1,0,-1,-1,0,0,-1,0,-1,0,0,1,-1,0,-1,0,0,0,-1,1,0,0,-1,0,-1,1,-1,-1,0,0,0,1,-1,0,0,1,-1,1,-1,0,0,0,-1,0,0,0,0,0,-1,1,0,1,0,0,-1,1,-1,0,0,1,0,1,-1,0,0,1,-1,0,-1,0,0,0,0,1,-1,0,1,0,-1,1,0,0,0,1,-1,1,0,0,0,0,0,1,-1,0,0,1,-1,1,-1,1,1,0,-1,0,-1,1,0,0,-1,1,0,0,0,0,0,1,0,0,0,0,-1,1,-1,-1,0,1,-1,1,0,0,1,1,-1,1,-1,1,0,0,0,1,0,0,0,0,-1,1,-1,1,0,1,0,1,-1,0,0,1,0,0,-1,0,1,0,-1,1,0,1,0,0,-1,1,0,0,0,0,-1,0,-1,1,0,1,0,1,0,0,1,1,-1,0,-1,0,1,1,-1,1,-1,0,0,0,0,1,0,0,0,0,0,1,-1,0,0,0,0,1,0,0,0,1,0,1,-1,1,1,0,-1,1,-1,1,1,1,-1,1,0,0,0,1,-1,0,-1,0,-1,0,0,1,0,1,0,1};

void WriteParamFile(int frameno, double param);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k;
double xstt = 0.25;
double xend = 1.25;
double xpos;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}
return 0;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j;

double specs[spcno];

double mandel=2;
double scale=15;
double screenX=200;
double screenY=0;
double cx=0;
double cy=0;
double altgamma=0;
double thisfn=4;
double split=1;
double orbitTrapE=0;
double addc=1;
double escapeonly=0;
double colsch=0;
double zetaterms=100;
double escapeboundExp=1.7;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=0;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=0;
double xm=0;
double xn=2;
double flip=1;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=zetaterms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip;

specs[29]=terms;
//specs[30]=-998; //Gives factorizations eta zeta 
specs[30]=0; //Gives factorizations eta 

for (j=1;j<=terms;j++)
	{
//    if(j % 2==1)
		specs[31+2*(j-1)]=factriz[j-1];
//	else
//		specs[31+2*(j-1)]=factriz[j-1]-2*factriz[j/2-1];
//    specs[31+2*j]=-pow(-1,factriz[j])*factriz[j]; //alternating version
//    specs[31+2*j]=-pow(-1,factriz[j])*factriz[j]; //alternating by factor number
	specs[32+2*(j-1)]=0;
	}

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


