//Generates a Dirichlet series function based on its product coefficients
//to be processed with RZViewer
//To run this file from terminal put it in a folder, cd to the folder and input
//gcc -o rzmovieo productcoeffs.c -lm && ./rzmovieo
//To run successive compiled versions type ./rzmovieo
//See: http://www.dhushara.com/DarkHeart/RZV/RZViewer.htm

#include <unistd.h>
#include <stdlib.h> 
#include <stdio.h>
#include<complex.h>
#include<math.h>


const double pi=3.1415926535;
const int frames = 1; //number of movie frames - use n+1 for circular
const int spcno = 29+2+2*1024;
const char base[] = "lambda";
double _Complex mcoefs[1024];
long *myp;
const int terms=1024;
int factors[20];

void WriteParamFile(int frameno, double param);
void prims(double terms);
int fact(int num);

int main(void)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int filetyp = 3;
int k,f;
double xstt = 0.25;
double xend = 1.25;
double xpos;
double golden;
sprintf(filename, "%s.txt", base);
mf=fopen(filename,"w");
fprintf(mf,"%f %f \n",(float)filetyp, (float)frames);
fclose(mf);
xpos=xstt;
myp = calloc((long)terms,sizeof(long)); 
prims(terms);
/*for(k=0;k<frames;k++)
	{
	if(k==0)
		xpos=xstt;
	else
		xpos=xstt+(xend-xstt)*k/(frames-1);
	WriteParamFile(k, (double)xpos);
	}*/
golden=(-1.0+sqrt(5))/2.0;
for(k=0;k<terms;k++)	
	mcoefs[k]=-1; //lambda = prod(1+p^-z)^-1
	//mcoefs[k]=cexp(k*I*golden/pi); //golden mean angle vaariation
WriteParamFile(0,0);
free(myp);
return 0;
}

void prims(double terms)
	{
	long j,k,pp,pt;
	double q;
	double myzt;
	*myp=2;
	//printf("%d ",*(myp));
	myzt=(long)terms;
	for (k=1;k<=myzt;k++)
		{
		pp=*(myp+k-1)+1;
		pt=0;
		j=1;
		while ((pt<k)&&(*(myp+pt)<=sqrt(pp)))
			{
			j=j+1;
			q=(double)pp/(*(myp+pt));
			if (q!=(long)q)
				pt++;
			else
				{
				pp++;
				pt=0;
				}
			}
		*(myp+k)=pp;
		//printf("%d ",*(myp+k));
		}
	//printf("\n");
	}
	
int fact(int num)
{
int ppos=0;
int fpos=0;
int p;
while (num>1)
	{
	p=*(myp+ppos);
	if (num % p == 0)
		{
		factors[fpos]=p;
		fpos++;
		num=num/p;
		}
	else
		{
		ppos++;
		p=*(myp+ppos);
		}
	}
	return fpos;
}

void WriteParamFile(int frameno, double param)
{
FILE *mf;
char filename [ FILENAME_MAX ];
int j,k,m,p,f;
double _Complex h;

double specs[spcno];

double mandel=2;
double scale=15;
double screenX=200;
double screenY=0;
double cx=0;
double cy=0;
double altgamma=0;
double thisfn=4;
double split=1;
double orbitTrapE=0;
double addc=1;
double escapeonly=0;
double colsch=0;
double terms=100;
double escapeboundExp=1.7;
double attractorEscBd=-2;
double maxIterations=64;
double dolog=0;
double sang=1;
double domoib=0;
double cval=0;
double oldfn=thisfn;
double realk=0;
double imagk=0;
double crval=0;
double fullt=0;
double xm=0;
double xn=2;
double flip=1;

specs[0]=mandel;
specs[1]=scale;
specs[2]=screenX;
specs[3]=screenY;
specs[4]=cx;
specs[5]=cy;
specs[6]=altgamma;
specs[7]=thisfn;
specs[8]=split;
specs[9]=orbitTrapE; //orbit trap epsilon)
specs[10]=addc; //additive c/ mult c
specs[11]=escapeonly;
specs[12]=colsch;
specs[13]=escapeboundExp; //escape bound exponent
specs[14]=terms;
specs[15]=attractorEscBd; //attractor escape bound 10^-k
specs[16]=maxIterations; //iterations
specs[17]=dolog; //take log of the function
specs[18]=sang; //angle ploting
specs[19]=domoib; //mobius
specs[20]=cval; 
specs[21]=oldfn; 
specs[22]=realk; 
specs[23]=imagk; 
specs[24]=crval; 
specs[25]=fullt; 
specs[26]=xm;
specs[27]=xn; 
specs[28]=flip;

specs[29]=1024;
//specs[30]=-998; //Gives factorizations eta zeta 
specs[30]=0; //Gives factorizations eta 

for (j=1;j<=1024;j++)
	{
	f=fact(j);
	h=1;
	for (k=0;k<f;k++)
		{
		m=0;
		while (*(myp+m)!=factors[k])
			m++;
		h=h*mcoefs[m];
		}
	//printf("%d,",(int)creal(h));
	specs[31+2*(j-1)]=creal(h);
	specs[32+2*(j-1)]=cimag(h);
	}

sprintf(filename, "%s%d.txt", base, frameno);
//printf("filename = \"%s\"\n", filename);
mf=fopen(filename,"w");
for (j=0;j<spcno;j++)
    fprintf(mf,"%f ",specs[j]);
fclose(mf);
}


