// compute one scan line of mandelbrot set

#include "mandel.h"
#include <complex.h>
#include <math.h>
#include <string.h>
#include <stdlib.h> 

double mystep; 
int myzt, altgam, thisfn, mdz, mft,mymw,mymh,myml,myxi,esco,orbto,mmel;
const double pi=3.1415926535;
double sq=1.7724538509;
double mtf=5.0;
const double tesc=0.001;
long *mymp;
double _Complex *mymv, *mymfl;
double split=0.0;

double _Complex Pgamma(register double _Complex z)
// gamma using product formula
{
int n,its;
double _Complex f, hold;

f=1/z;
its=(int)pow(10,altgam);
for (n=1; n<=its; n++ )
	{
	hold=cpow(1.0+1.0/n,z)/(1.0+z/n);
    f=f*hold;
	if (cabs(hold-1.0)<tesc)
		break;
	}
return f;
}

double _Complex Lgamma(register double _Complex z)
//complex gamma function using Lanczos approximation to extend to the entire plane. 
{
	double _Complex zz, t, LogofGamma, s, ss, f;

	double twopi, fac;
	double c[12];
	int k,g;
	
	if (altgam)
		f=Pgamma(z);
	else
		{

		c[1] = 1.000000000000000174663;
		c[2] = 5716.400188274341379136;
		c[3] = -14815.30426768413909044;
		c[4] = 14291.49277657478554025;
		c[5] = -6348.160217641458813289;
		c[6] = 1301.608286058321874105;
		c[7] = -108.1767053514369634679;
		c[8] = 2.605696505611755827729;
		c[9] = -0.7423452510201416151527e-2;
		c[10] = 0.5384136432509564062961e-7;
		c[11] = -0.4023533141268236372067e-8;
		twopi=pi+pi;
		g=9;
		s=0;
		zz=z;
		if ((round(creal(zz))==creal(zz))&&(cimag(zz)==0)&&(creal(zz)<=0))
			{
			f=10e308;
			}
		if ((round(creal(zz))==creal(zz))&&(cimag(zz)==0)&&(creal(zz)>=0))
			{
			fac=1;
			for (k=1; k<=creal(zz)-1; k++)
				fac=fac*k;
			f=fac;
			}
		if ((round(creal(zz))!=zz)||(cimag(zz)!=0))
			//else
			{
			if (creal(z)<0)
				z=-z;
			t=z+g;
			for ( k = g+2; k>=2; k-- ) 
				{
				s=s+c[k]/t;
				t=t-1;
					}
			s=s+c[1];
			ss=(z+g-0.5);
			s=clog(s*sqrt(twopi))+(z-0.5)*clog(ss)-ss;
			LogofGamma = s;
			f = cexp(LogofGamma);
			if (creal(zz)<0)
				{
				f=-pi/(z*f*csin(pi*z));
				}
		} }
	
	return f;
}

double _Complex zetaa(register double _Complex z)
	{
		double _Complex zz, hold, hold2, f;
		int steps, sw, i;
		double mymod;
		//finds Riemann zeta and direct derivatives 

		zz=z;
		if (creal(z)>100)
			{
			f=1;
			return f;
			}
		else
		{
		if (creal(z)<split)
			z=1-z;
		//steps=2002;
		steps=myzt;
		f=0;
		sw=-1;
		for (i = 1; i<=steps; i++ ) 
		{
			sw=-sw;
			hold2=cpow(i,z);//*log(i+1);
			if ((isinf(creal(hold2)))||(isinf(cimag(hold2))))
				hold=0;
			else
				hold=sw/hold2;
			f=f+hold;
			mymod=cabs(hold);
			if (!mft)
				if (mymod/cabs(f)<tesc) //changed to be relative to value of function
					break;
		}
	sw=-sw;
	hold2=cpow((i+1),z);//*log(i+2);
	if ((isinf(creal(hold2)))||(isinf(cimag(hold2))))
		hold=0;
	else
		hold=sw/hold2;	
	f=f+hold/2;
	hold2=cpow(2,(1-z));//0;
	if ((isinf(creal(hold2)))||(isinf(cimag(hold2))))
		hold=0;
	else
		hold=1/(1-hold2);	

	//f=f/(1-cpow(2,(1-z)));
	f=f*hold;
	if(creal(zz)<split)
		{
			f=2*cpow(2*pi,-z)*ccos(z*pi/2)*Lgamma(z)*f;
			//f=cpow(2,1-z)*cpow(pi,-z)*csin(pi*(1-z)/2)*Lgamma(z)*f; //alternative formula
		}
if (myxi==0)
	return f;
else
	if (*(mymv)==1+I)
		return zz/2*(zz-1)*Lgamma(zz/2)*cpow(pi,-zz/2)*f;
	else
		return cpow(pi,-zz/2)*Lgamma(zz/2)*f;

	}
	}

double _Complex zint(register double _Complex s)
{
	//local Mellin transform for Dirichlet L-fnctions
	int r;
	int evn;
	int m=myzt;//200;
	int k,h,j,n,q;
	double yj,yj2,ya,yb;
	double _Complex z,zh,zh2,zz,ep,ph;
	k=creal(*(mymv));
	h=cimag(*(mymv));
	double b=1.0/k;
	double t=10.0;
	if(k>1)
	{
		r=sqrt(m);//10;
		z=0;
		if(creal(*(mymv+(int)k-1))+1<0.1) evn=0; else evn=1;
		ph=0;
		for(n=1;n<k;n++) ph=ph+conj(*(mymv+n))*cexp(-2*pi*I*n/k);
		//for(n=1;n<k;n++) ph=ph+cexp(-2*pi*I*n/k)/(*(mymv+n)); //maybe need + here!
		if (evn) ep=cpow(k,1-s)/ph; else ep=-I*cpow(k,1-s)/ph; //-I*-I=-1
		for(j=1;j<=m-1;j++)
		{
			yj=t/(b+(t-b)*(m-j)/(m-1))/k; //method3 t=10 b=1/k;
			yj2=t/(b+(t-b)*(m-j-1)/(m-1))/k;	
			//ya=(yj+yj2)/2;
			ya=2.0/(1.0/yj+1.0/yj2);		
			zh=0;
			q=1;
			for(n=1;n<=r;n++)
				{
				if (*(mymv+q)!=0)
					if(evn==1)
						zh=zh+(*(mymv+q))*exp(-pi*(n*n)*ya);
					else
						zh=zh+(*(mymv+q))*n*sqrt(ya)*exp(-pi*(n*n)*ya);
				q=q+1;
				if (q==k+1)	q=1;				
				}
			zh2=0;
			q=1;
			for(n=1;n<=r;n++)
				{
				if (*(mymv+q)!=0)
					if(evn==1)
						//zh2=zh2+(*(mymv+q))*exp(-pi*(n*n)*yb);
						zh2=zh2+(conj(*(mymv+q)))*exp(-pi*(n*n)*ya);
					else
						//zh2=zh2+(*(mymv+q))*n*sqrt(yb)*exp(-pi*(n*n)*yb);
						zh2=zh2+(conj(*(mymv+q)))*n*sqrt(ya)*exp(-pi*(n*n)*ya);
				q=q+1;
				if (q==k+1)	q=1;					
				}
			//zh2=-I*zh2*ph/k/sqrt(ya);
			zz=(cpow(ya,s/2)*zh+ep*cpow(ya,(1-s)/2)*zh2)/ya*(yj2-yj);
			//zz=(ep*cpow(ya,(1-s)/2)*zh2)/ya*(yj2-yj);
			//zz=(cpow(ya,s/2)*zh)/ya*(yj2-yj);
			z=z+zz;
		}
		//z=z*cpow(pi,(s+evn)/2)/Lgamma((s+evn)/2)/cpow(k,s/2);
		//z=z*cpow(pi,(s+1)/2)/Lgamma((s+1)/2)/cpow(k,s/2);
		if (evn==1)
			z=z*cpow(pi,s/2)/Lgamma(s/2);
		else 
			z=z*cpow(pi,(s+1)/2)/Lgamma((s+1)/2);
		return z;
	}
}

			
double _Complex zeta(register double _Complex z)
	{
		//Generalized zeta for Riemann zeta and related and Dirichlet L-functions
		double _Complex zz, hold, hold2, f;
		int steps,i,j,a,flg,hh,g,p;
		double mymod;
		double k,h;
		//double split=0.0;
		double _Complex t;
		
		k=creal(*(mymv));
		h=cimag(*(mymv));
		//printf("%f ",k);
	
		if (k==1)
			{
			if ((h==1)||(h==0))
				{z=zetaa(z); return z;}
			if (h==-1)
				{z=zetaa(z)*(1-cpow(2,(1-z))); return z;}
			if (h<-1)
				{
				f=zetaa(z);
				hh=-h;
				//printf("%d ",hh);
				j=0;
				//printf("%d ",*(mymp+(int)j));
				p=*(mymp+(int)j);
				while ((p>0)&&(p<=hh))
					{			
					if((int)hh % p==0)
						{
						f=f*(1-cpow(p,-z));
						}
					j=j+1;
					p=*(mymp+(int)j);
					}
					//f=f*(1+cpow(2,-zz))*(1+cpow(5,-zz));
				return f;
				}
			}
				
	else
				
		{		
		if(creal(*(mymv+(int)k-1))+1<0.1)
		//if(*(mymv+(int)k-1)==-1)
			a=1;
		else
			a=0;
		zz=z;
		if (creal(z)>100.0)
			{
			f=1;
			return f;
			}
		else
		{
		if (creal(z)<split)
			z=1-z;
		steps=k*(int)(myzt/k)-1;
		f=0;
		j=1;
		flg=(creal(zz)<split);
		for (i = 1; i<=steps; i++ ) 
		{
			//printf("%d ",sw*j);
			if (*(mymv+j)!=0)
				{
				hold2=cpow(i,z);
				if ((isinf(creal(hold2)))||(isinf(cimag(hold2))))
					hold=0;
				else
					if (flg)
						hold=conj(*(mymv+j))/hold2;
					else
						hold=*(mymv+j)/hold2;
				if (i!=steps)
					f=f+hold;
				else
					f=f+hold/2;				
				mymod=cabs(hold);
				if (!mft)
					if (mymod/cabs(f)<tesc) //changed to be relative to value of function
						break;
				}
				j=j+1;
				if (j==k+1)
					j=1;
		
		}
			//f=cpow(pi/k,-(z+a)/2)*Lgamma((z+a)/2)*f;
	if(creal(zz)<split)
		{
			t=0.0;
			for (i = 1; i<=k; i++ )
				t=t+cexp(2.0*i*I*pi/k)*(*(mymv+i));
			//printf("%d ",a);
			f=cpow(k/2.0/pi,z)/k*t*Lgamma(z)*(cexp(-pi*I*z/2.0)+(*(mymv+(int)k-1))*cexp(pi*I*z/2.0))*f;
			//f=cpow(pi/k,-(z+a)/2)*Lgamma((z+a)/2)*f; //alternative equivalent formula
			//f=f*cpow(I,a)*sqrt(k)/-conj(t);
			//f=cpow(pi/k,((1-z)+a)/2)/Lgamma(((1-z)+a)/2)*f;

		}
		}
	if(mmel)
		if((k<=63)&&(k>1))
			if((creal(zz)<1.2)&&(creal(zz)>-0.15)&&(cimag(zz)<7)&&(cimag(zz)>-7))		
				f=zint(zz);
		if(h<0)
		{
		hh=-h;
		//printf("%d ",hh);
		j=0;
		//printf("%d ",*(mymp+(int)j));
		p=*(mymp+(int)j);
		while ((p>0)&&(p<=hh))
			{			
			if((int)hh % p==0)
				{
				g=p;
				f=f*(1-(*(mymv+(g % (int)k))*cpow(p,-zz)));
				}
			j=j+1;
			p=*(mymp+(int)j);
			}
			//f=f*(1+cpow(2,-zz))*(1+cpow(5,-zz));
		}
			
if (myxi==0)
	return f;
else
	if(creal(*(mymv+(int)k-1))+1<0.1)
		a=1;
	else
		a=0;
	return cpow(pi/k,-(zz+a)/2)*Lgamma((zz+a)/2)*f;
	}
}	
	
double _Complex rawint2(register double _Complex s)
{
	//Second local Mellin transform for abstract L-functions
	int m=myzt;
	int k,h,j,q,d,mpos,sh,r;
	double yj,yj2,ya,n,w,sn,ys,A;
	double _Complex z,zh,zz,ga,e,hh,a;
	k=creal(*(mymv));
	h=cimag(*(mymv));
	d=fabs(*(mymv+k+1));
	n=*(mymv+k+d+2);
	e=*(mymv+k+d+3);
	e=e/cabs(e);
	w=creal(*(mymv+k+d+4));
	sn=sqrt(n);
	//A=sn/pow(pi,0.5*d);
	if(m>k) m=k; //avoid overrunning coefficients
	double b=1.0;
	double t=mtf*sn;
	r=m;//sqrt(m);
	if(k>1)
	{
		if (h==-996) mpos=k+d+5; else mpos=k+d+4;					
		if (h!=-996) sh=7; else sh=8;	
		if (creal(*(mymv+k+1))<0) mpos=mpos+3;
		//printf("mp=%d ",mpos);
		z=0;		
        if (*(mymv+k+d+sh)!=0) //pole
            z=z-*(mymv+k+d+sh)*(1.0/(0-s)-1.0/(1-s))/cpow(n,1/2);  // cpow(n,s/2);
            //z=z+*(mymv+k+d+sh)*(1.0/(0-s)-1.0/(1-s))/cpow(n,s/2);
            //z=z+1.0/4*(1/(s-1)-1/s);
        for(j=1;j<=m-1;j++)
			{
			yj=t/(b+(t-b)*(m-j)/(m-1));
			yj2=t/(b+(t-b)*(m-j-1)/(m-1));	
			ya=t/(b+(t-b)*(m-j-0.5)/(m-1));
				ys=ya/sn;
			zh=0;
			for(q=1;q<=r;q++)
				{
				if (*(mymv+q)!=0)
					{
					hh=(*(mymv+q))*creal(*(mymv+mpos+m*(j-1)+q));
					//hh=(*(mymv+q))*exp(-2*pi*q*ya/sn);
					zh=zh+hh;
					if (cabs(hh)/cabs(zh)<0.0001) break;
					}
				}
				zz=(cpow(ys,s)+e*cpow(n,0.5*w-s)*cpow(ys,w-s))*zh/ya*(yj2-yj);
			z=z+zz;
			}
		//z=z*ns;
		ga=1;
		a=1;
		a=a*2.0*pow(pi,0.25*d);	
		for(j=1;j<=d;j++)
		{
			ga=ga*Lgamma((s+*(mymv+k+j+1))/2);
			a=a*cpow(pi,-(s+*(mymv+k+j+1))/2);
		}
		z=z/a/ga;
		return z;
	}
}


double _Complex rawint(register double _Complex s)
{
//First local Mellin transform for abstract L-functions	
int r;
int m=myzt;//200;

int k,h,j,q,d,sh;
double yj,yj2,ya,n,w,sn;
double _Complex z,zh,zz,ep,ph,e,hh;
k=creal(*(mymv));
h=cimag(*(mymv));
d=fabs(*(mymv+k+1));
n=*(mymv+k+d+2);
e=*(mymv+k+d+3);
e=e/cabs(e);
w=creal(*(mymv+k+d+4));
if(m>k) m=k; //avoid overrunning coefficients
sn=sqrt(n);
double b=1.0;
double t=5.0*sn;
	r=m;//sqrt(m);
if(k>1)
	{
	z=0;
	for(j=1;j<=m-1;j++)
		{
		yj=t/(b+(t-b)*(m-j)/(m-1));
		yj2=t/(b+(t-b)*(m-j-1)/(m-1));	
		//ya=(yj+yj2)/2;
		ya=2.0/(1.0/yj+1.0/yj2);		
		zh=0;
		for(q=1;q<=r;q++)
			{
			if (*(mymv+q)!=0)
				{
				hh=(*(mymv+q))*exp(-2.0*pi*q*ya/sn);
				zh=zh+hh;
				if (cabs(hh)/cabs(zh)<0.0001) break;
				}
			}
		zz=(cpow(ya/sn,s)+cpow(n,w/2-s)*e*cpow(ya/sn,w-s))*zh/ya*(yj2-yj);
		z=z+zz;
	}
//	if (h!=-996) sh=7; else sh=8;	
//	if (*(mymv+k+d+sh)!=0)
//		z=z+*(mymv+k+d+sh)*(1.0/(0-s)-1.0/(1-s))/cpow(n,s/2);
	z=z*cpow(2*pi,s)/Lgamma(s);
/*	if (h!=-996) sh=7; else sh=8;	
	if (*(mymv+k+d+sh)!=0)
		z=z+*(mymv+k+d+sh)/(1-s);*/
	return z;
	}
}

double _Complex zetar(double _Complex z)
{
	//Generalized zeta and L-function routine
	int j,k,h,m,d,twst,modular,mz;
	double mymod,n,w,sh,ht,wd;
	double _Complex zz,zh,zm,ga,gb,a,b,e,ee,zn;
	zz=0.0;
	k=creal(*(mymv));
	h=cimag(*(mymv));
	m=1;
	zm=z;
	zn=z;
	mz=myzt;
	if(mz>k)mz=k;
	if((h>=-997)&&(h<=-995))
		{
		d=fabs(*(mymv+k+1)); //no of gamma factors
		n=*(mymv+k+d+2); //conductor
		e=*(mymv+k+d+3); //reflection factor*scal conductor
		ee=cabs(e); //scal conductor
		e=e/ee; //reflection factor
		w=creal(*(mymv+k+d+4)); //weight
		if((d==2)&&(*(mymv+k+2)==0)&&(*(mymv+k+3)==1)&&(w>=2)) modular=1; else modular=0; //modular L-function
		ht=0; wd=0; //set Mellin window size
			if (modular==1) 
			{ 
				ht=8-5*n/150000; 
				if (ht<0) ht=0;
				wd=1;//w/2;
			}
		if(creal(*(mymv+k+1))<0)
			if (h!=-996)
				{ 
				ht=creal(*(mymv+k+d+5)); 
				wd=creal(*(mymv+k+d+6)); 
				}
			else
				{ 
				ht=creal(*(mymv+k+d+6)); 
				wd=creal(*(mymv+k+d+7)); 
				}
		//e=cexp(pi*I*(1-e)/2);
		twst=(int)cimag(*(mymv+k+d+4)); //twst sets pi factors to gamma values
		sh=0;
		if (h==-996)
			sh=*(mymv+k+d+5); //shift factor for large weights	
		if(creal(z)<w*split)// if z<crit split reflect in crit line and shift if necc
			z=w-z-sh;
		else
			z=z-sh;
		}
	for (j=1;j<=mz;j++)
		{
		if(k==1)
		{
			if(h==-1)
				zh=pow(-1,j-1)*cpow(j,-z);
			else
				zh=cpow(j,-z);
			if (isnan(zh))
			 break;
			
		zz=zz+zh;
		
		mymod=cabs(zh);
		if (!mft)
			if (mymod/cabs(zz)<tesc) //changed to be relative to value of function
				break;
		}
		else
		 if (*(mymv+m)!=0) //calculate Dirichlet series term
			{
			if(h==-999)
				zh=(*(mymv+m))*cpow(*(mymv+k+m)+j,-z); //shifted term -996
			else
				{
				if((creal(zn)<w*split)&&(h>=-997)&&(h<=-995))
					zh=(conj(*(mymv+m)))*cpow(j,-z);  //conjugate term to the left
				else
					zh=(*(mymv+m))*cpow(j,-z);		//normal term to the right		
				}
			
		    if (isnan(zh)) //break if term not a number without using it
			 break;			
			zz=zz+zh;
		
			mymod=cabs(zh);
			if (!mft) // if not full terms
				if (mymod/cabs(zz)<tesc) //cutoff relative to value of function
					break;		}	
			m=m+1; //iterate m periodically i it exceeds k
			if(m==k+1)
				m=1;
			}
	if (h==-998) // we now make z the range (output) variable
		z=zz/(1-cpow(2,(1-z))); //male an eta version??
	else
		z=zz;
	if ((h>=-997)&&(h<=-995)&&(creal(zm)<w*split)) //if to the left pass through the functional equation
		{
		ga=1;  //right-hand gammas and pis
		zm=w-zm;
		a=cpow(n,zm/2);
		for(j=1;j<=d;j++)
		{
			ga=ga*Lgamma((zm+*(mymv+k+j+1))/2);
			if (twst==0)
				a=a*cpow(pi,-zm/2);
			else
				a=a*cpow(pi,-(zm+*(mymv+k+j+1))/2);
		}
		z=a*ga*z;
		z=e*z;  //reflection factor
		gb=1; // left-hand gammas and pis
		b=cpow(n,(w-zm)/2);
		for(j=1;j<=d;j++)
		{
			gb=gb*Lgamma((zn+conj(*(mymv+k+j+1)))/2);
			if (twst==0)
			b=b*cpow(pi,-(w-zm)/2);
		else
			b=b*cpow(pi,-((w-zm)+conj(*(mymv+k+j+1)))/2);
		}
	z=z/b/gb;		
		}
if(mmel) // if mmel selected do a Mellin transform window
	if((h>=-997)&&(h<=-995))
		if((creal(zn)<w/2+wd)&&(creal(zn)>w/2-wd)&&(cimag(zn)<ht)&&(cimag(zn)>-ht))		
			if(modular==1)
				z=rawint(zn);
			else
				z=rawint2(zn);
if ((myxi==0)||(h<-997)||(h>-995))  // an abstrct L-function with gammas
	if ((h>=-997)&&(h<=-995)&&(ee!=1))
		return cpow(ee,-zn)*z;  //scaled for conductor
	else
		return z; //normal
else  //this is a xi plot and we need to apply gammas
	if (creal(zn)<w*split)
		return e*b*gb*z;
	else
		{
		ga=1;
		a=cpow(n,zn/2);
		//zm=w-zm;
		for(j=1;j<=d;j++)
			{
			ga=ga*Lgamma((zn+(*(mymv+k+j+1)))/2);
			if (twst==0)
				a=a*cpow(pi,-zn/2);
			else
				a=a*cpow(pi,-(zn+*(mymv+k+j+1))/2);
			}
		return a*ga*z;
		}
}

double _Complex zetap(double _Complex z)
{
	//zeta using the Euler product
	int j,k,m,prm,h,hh,p,g;
	//double cutoff=0.000001;
	double _Complex zz,zh;
	long pr;
	zz=1.0;
	//myzt=33547;	
	//myzt=84270;
	//myzt=50;
	//z=0.5+14.13472514*I;
	/*	if (creal(z)<0)
	 {
	 return 0;
	 }
	 else*/
	{
		k=(int)(creal(*mymv));
		h=(int)(cimag(*mymv));
		m=1;
		for (j=0;j<myzt;j++)	
		{
			//	if (*(mymp+j)==0)
			//		printf("# %d ",j);
			//zh=(1-cpow(*(mymp+j),-z));
			pr=*(mymp+j);
			//prm=(int)(((double)pr)/m-pr/m)*m;
			prm=pr % m;		
			//printf("%d %d %d\n",pr,m,prm);
			if(k==1)
				zh=(1-cpow(*(mymp+j),-z));
			else
				zh=(1-(*(mymv+prm+1))*cpow(*(mymp+j),-z));
			if ((isnan(zh))||(zz==0))//||(cabs(zh-1)<cutoff))
				break;
			zz=zz/zh;		
			m=m+1;
			if(m==k+1)
				m=1;
		}
		if(h<0)
		{
			hh=-h;
			j=0;
			p=*(mymp+(int)j);
			while ((p>0)&&(p<=hh))
			{			
				if((int)hh % p==0)
				{
					g=p;
					if (k==1)
						zz=zz*(1-cpow(p,-z));
					else
						zz=zz*(1-(*(mymv+(g % (int)k))*cpow(p,-z)));
				}
				j=j+1;
				p=*(mymp+(int)j);
			}
		}
		
		z=zz;
		//printf("%f\n",cabs(z));	
		return z;
	}
}

double _Complex przeta(double _Complex z)
{
	//prime zeta function
	long j,k,m,n,o;
	double _Complex zz,zh;
	zz=0;
	if (creal(z)<0)
		{
		return 0.0001*I;
		}
	for (k=1;k<=myzt;k++)
		{
		m=1;
		n=k;
		for (j=0;j<myzt;j++)
			{
			o=*(mymp+j);
			if (o==0)
				{
				//printf("# %d ",j);
				break;
				}
			if ((n % o ==0)&&(n % (o*o) !=0))
				m=-m;
			if (n % (o*o) ==0)
				{
				m=0;
				break;
				}	
			}
		if (m!=0)
			{
			zh=clog(zeta(k*z));
			if (isnan(creal(zh)))
				{
				zz=zh;
				break;
				}
			//zz=zz+(double)m/k*zh;
			zz=zz+zh*m/k;
			}
		}
	return zz;

}

double _Complex digamma(register double _Complex z)
{
double _Complex f=-0.57721566490153286060651209008240243;
int n;
double _Complex hold;

for (n=1;n<=myzt;n++)
	{
    hold=(z-1.0)/n/(n+z-1.0);
	f=f+hold;
    if ((cabs(hold)!=0)&&(cabs(hold)<0.001))
        break;
	}
return f;
}

double _Complex dgamma(register double _Complex z)
{
return Lgamma(z)*digamma(z);
}

double _Complex thdzeta(register double _Complex z)
//this is the theoretical dzeta and gives good values
{
double _Complex zz, f, ff, hold;
int sw, i;
double pi2, mymod;
double mycut=0;

zz=z;
if (creal(z)<mycut)
    z=1-z;
f=0;
sw=1;
for (i=1;i<=myzt;i++)
	{
    sw=-sw;
    hold=sw*log(i)/cpow(i,z);
    f=f+hold;
	mymod=cabs(hold);
    if ((mymod!=0)&&(mymod/cabs(f)<tesc)) //changed to be relative to value of function
        break;
	}
sw=-sw;
f=f+sw*log(i)/cpow(i,z)/2;
f=f-cpow(2,(1-z))*log(2)*zeta(z);
f=f/(1-cpow(2,(1-z)));
ff=f;
if (creal(zz)<mycut)
	{
    pi2=pi+pi;
    f=(clog(pi2)*csin(pi*zz/2)+pi/2*ccos(pi*zz/2))*Lgamma(z)*zeta(z);
 	f=f-csin(pi*zz/2)*Lgamma(z)*(digamma(z)*zeta(z)+ff);
    f=f*(cpow(pi2,zz))/pi;
	}
return f;
}

double _Complex dzeta(register double _Complex z)
//this is using differentials to calculate dzeta uless redirected
{
double _Complex h;
double myfrac=1.0/2000;
double k,m;

k=creal(*(mymv));
m=cimag(*(mymv));
//if((k==1)&&(m==1)&&(creal(z)>split)) 
//	z=thdzeta(z);
//else
{
if (creal(z)>1)
	myfrac=myfrac*creal(z);
h=mystep*myfrac*(1+1*_Complex_I);
z=(zeta(z+h)-zeta(z))/h;
}
return z;  
}	

double _Complex xi(register double _Complex z)
{
double k,h;
int a;//,i;
//double _Complex t,zz;
k=creal(*(mymv));
h=cimag(*(mymv));
if (*(mymv)==1+I)
//if(h==1)
	{
//	if(creal(z)<split)
//	if(creal(z)<0.5)
//		z=1-z;
	return z/2*(z-1)*Lgamma(z/2)*cpow(pi,-z/2)*zeta(z);
	//return Lgamma(z/2)*cpow(pi,-z/2)*zeta(z);
	}
else
{
if(creal(*(mymv+(int)k-1))+1<0.1)
	a=1;
else
	a=0;
//printf("%4.2f ",creal(*(mymv+(int)k-1)));
//zz=z;

//if (creal(zz)>split)
//if (creal(zz)>=0.5)
//a=1;
	z=cpow(pi/k,-(z+a)/2)*Lgamma((z+a)/2)*zeta(z);
/*else
	{
	t=0.0;
	for (i = 1; i<=k; i++ )
		t=t+cexp(2.0*i*I*pi/k)*(*(mymv+i));
	z=1-z;
	z=cpow(pi/k,-(z+a)/2)*Lgamma((z+a)/2)*zeta(z,0);
	if (t!=0)
		z=z*(cpow(i,a)*sqrt(k)/t);
	}*/
return z;
//return Lgamma(z/2+1)*(z-1)*cpow(pi,-z/2)*zeta(z,1);
//return z/2*(z-1)*Lgamma(z/2)*cpow(pi,-z/2)*zeta(z,1);
//return Lgamma(z/2)*cpow(pi,-z/2)*zeta(z,1);

}
}

double _Complex dxi(register double _Complex z)
{
double k,h, a;

k=creal(*(mymv));
h=cimag(*(mymv));
if(creal(*(mymv+(int)k-1))+1<0.1)
		a=1;
	else
		a=0;
if(k==1)
{
//if (creal(z)<0.5)
//	{
//	z=1-z;
	//return -xi(z)*(dgamma(z/2+1)/2/Lgamma(z/2+1)+1/(z-1)-log(pi)/2+dzeta(z)/zeta(z,1));
//	}
//else
	return  xi(z)*(dgamma(z/2+1)/2/Lgamma(z/2+1)+1/(z-1)-log(pi)/2+dzeta(z)/zeta(z));
}
else
{
//if (creal(z)<0.5)
//	{
//	z=1-z;
//	return -xi(z)*(dgamma((z+a)/2)/2/Lgamma((z+a)/2)-log(pi/k)/2+dzeta(z)/zeta(z,1));
//	}
//else
	return  xi(z)*(dgamma((z+a)/2)/2/Lgamma((z+a)/2)-log(pi/k)/2+dzeta(z)/zeta(z));
}
}
double _Complex theta(register double _Complex z)
{
z=-I*(z-1/2);
z=(clog(Lgamma((2*I*z+1)/4))-clog(Lgamma((-2*I*z+1)/4)))/2/I-log(pi)*z/2;
//z=carg(Lgamma((2*I*z+1)/4))-log(pi)*z/2;
return z;
}
double _Complex rsz(register double _Complex z)
{
//z=cexp(I*theta(z))*zeta(1/2+I*z);
//z=cexp(I*theta(-I*(z-1/2)))*zeta(z,1);
z=cexp(I*theta(z))*zeta(z);
return z;
}

double _Complex dedint(register double _Complex s)
{
int m=myzt;//200;
int r=sqrt(m);
int k,j,n,q;
double t=10.0;
double b=1.0;
//double p=5;
//t=1; //method 1
//t=log(2)/log(r);   //method2
//b=0;
double yj,yj2,ya;
double _Complex z,zh,zz;
k=(int)(fabs(creal(*mymv))); 
if((k==1)||(k==0))
{
z=1.0/4*(1/(s-1)-1/s);
for(j=1;j<=m-1;j++)
	{
	yj=t/(b+(t-b)*(m-j)/(m-1)); //method3 harmonic
	yj2=t/(b+(t-b)*(m-j-1)/(m-1));
//	yj=b*pow(t/b,(j-1)/(m-1));	//method 4 exponential
//	yj2=b*pow(t/b,j/(m-1));
//	yj=pow(p,(b+(t-b)*(j-1)/(m-1)));	//method 1 logarithmic
//	yj2=pow(p,(b+(t-b)*j/(m-1)));
//	yj=pow(p,pow(p,(b+(t-b)*(j-1)/(m-1)))-1); //method2 double log
//	yj2=pow(p,pow(p,(b+(t-b)*j/(m-1)))-1);
	//ya=(yj+yj2)/2;
	ya=2.0/(1.0/yj+1.0/yj2);
	//ya=sqrt(yj*yj2); 
    zh=0;
    for(n=-r;n<=r;n++) 
        zh=zh+exp(-pi*n*n*ya);
    zh=zh*zh;    
    zz=(cpow(ya,s)+cpow(ya,(1-s)))*(zh-1)/4/ya*(yj2-yj);
    z=z+zz;
	}
z=z*cpow(pi,s)/Lgamma(s);
return z;
}
else
{
r=4.0/3*pow(m,0.25);
z=0;
for(j=1;j<=m-1;j++)
	{
	yj=t/(b+(t-b)*(m-j)/(m-1)); //method3 t=10 b=1;
	yj2=t/(b+(t-b)*(m-j-1)/(m-1));	
//	yj=pow(p,(b+(t-b)*(j-1)/(m-1)));
//	yj2=pow(p,(b+(t-b)*j/(m-1)));
//	yj=pow(p,pow(p,(b+(t-b)*(j-1)/(m-1)))-1);
//	yj2=pow(p,pow(p,(b+(t-b)*j/(m-1)))-1);
    ya=(yj+yj2)/2;
    zh=0;
    for(n=0;n<=r;n++)
      for(q=0;q<=r;q++)
		if((n!=0)||(q!=0))
	//	if(k>0) //k set to abs (above) because L(-k)=L(k)
			if((n!=0)||(q!=0))
			if((n!=0)&&(q!=0))
			zh=zh+4.0*creal(cpow((n+I*q),2.0*k))*exp(-pi*(n*n+q*q)*ya);
			else
			zh=zh+2.0*creal(cpow((n+I*q),2.0*k))*exp(-pi*(n*n+q*q)*ya);
	//	else
	//		zh=zh+cpow((n-I*q),-2.0*k)*exp(-pi*(n*n+q*q)*ya);
	zh=zh*cpow(ya,k);
    zz=(cpow(ya,s)+pow(-1,k)*cpow(ya,(1-s)))*zh/4/ya*(yj2-yj);
    z=z+zz;
	}
z=z*cpow(pi,s+fabs(k))/Lgamma(s+fabs(k));
return z;
}
}

double _Complex ded(register double _Complex z)
{
double _Complex zh,zz,az,hold;
int sq,m,n,k;
double  nz,mymod;
k=(int)(creal(*mymv)); 
//printf("%d ",k);
if((k==1)||(k==0))
{
zh=0.0;
zz=z;
sq=(int)(sqrt(myzt));
if (creal(z)<split)
	z=1-z;
/*for (m=0;m<=sq;m++)
	for (n=1;n<=sq;n++)
		{
		nz=m*m+n*n;
		hold=cpow(nz,-z);
		if ((m!=0)||(n!=0))
			zh=zh+hold+hold;			
		mymod=cabs(hold);*/
for (m=0;m<=sq;m++)
	for (n=m;n<=sq;n++)
		{
		nz=m*m+n*n;
		hold=2*cpow(nz,-z);
		if ((m!=0)||(n!=0))
		if ((m<n)&&(m>0))
			zh=zh+hold+hold;
		else
			zh=zh+hold;
		mymod=cabs(hold);
		
		if (!mft)
			if (mymod/cabs(zh)<tesc) //changed to be relative to value of function
				break;
		}
if(creal(zz)<split)
	zh=cpow(pi,1-2*z)*Lgamma(z)/Lgamma(1-z)*zh;

if(mmel)
	if((creal(zz)<1.8)&&(creal(zz)>-0.3)&&(cimag(zz)<7.5)&&(cimag(zz)>-7.5))		
	zh=2.0*dedint(zz);
z=zh/2;
if (myxi==0)
	return z;
else
	return cpow(pi,-(zz))*Lgamma(zz)*z;

}
else
{
zh=0;
zz=z;
sq=(int)(sqrt(myzt));
if (creal(z)<split)
	z=1-z;
/*for (m=0;m<=sq;m++)
	for (n=1;n<=sq;n++)
		{
		az=m+I*n;
		nz=m*m+n*n;
		hold=cpow(az/conj(az),k)*cpow(nz,-z);
		if ((m!=0)||(n!=0))
			zh=zh+hold+hold;
//		if ((m!=0)&&(n!=0))
//			zh=zh+hold;
		mymod=cabs(hold);	*/	
for (m=0;m<=sq;m++)
	for (n=m;n<=sq;n++)
		{
		az=m+I*n;
		nz=m*m+n*n;
		hold=cpow(az/conj(az),k)*cpow(nz,-z);
		if ((m!=0)||(n!=0))
		if ((m<n)&&(m>0))
			zh=zh+4*creal(cpow(az/conj(az),k))*cpow(nz,-z);
		else
			zh=zh+2*cpow(az/conj(az),k)*cpow(nz,-z);
		mymod=cabs(hold);
		if (!mft)
			if (mymod/cabs(zh)<tesc) //changed to be relative to value of function
				break;
		}
if(creal(zz)<split)
	//zh=pow(-1,k)*cpow(pi,-z-fabs(k))/cpow(pi,-1+z-fabs(k))*Lgamma(z+fabs(k))/Lgamma(1-z+fabs(k))*zh;
	zh=pow(-1,k)*cpow(pi,1-2*z)*Lgamma(z+fabs(k))/Lgamma(1-z+fabs(k))*zh;
if(fabs(k<10))
 if((creal(zz)<1.5)&&(creal(zz)>-0.2)&&(cimag(zz)<8)&&(cimag(zz)>-8))		
	zh=2.0*dedint(zz);
z=zh/2;		
if (myxi==0)
	return z;
else
	return pow(-1,k)*cpow(pi,-(zz))*Lgamma(zz+fabs(k))*z;
}
}

double _Complex daven(register double _Complex z)
{
double _Complex zh,zz,z1,z2,hold,w;
int m,n,k,h,i;
int thou=1000;
double  mymod,v,hh,tn,t;
double v3[3]={1,-1,0};
double v4[4]={1,0,-1,0};
double si=(sqrt(10.0-2.0*sqrt(5.0))-2.0)/(sqrt(5.0)-1.0);
double th=atan2(si,1.0);
double v51[5]={1,si,-si,-1,0};
double v52[5]={1,-1.0/si,1.0/si,-1,0};
double _Complex v53[5]={1,I,-I,-1,0};
double _Complex v54[5]={1,-I,I,-1,0};
double v55[5]={1,-1,-1,1,0};
double v6[6]={0,1,0,0,0,-1};
double al=0.80194;
double v7[7]={1,-(1+al),-al,al,1+al,-1,0};
double v8[8]={1,0,-1,0,1,0,-1,0};
k=(int)(creal(*mymv)); 
h=(int)(cimag(*mymv));
//printf("%d %f  ",h,cimag(*mymv)); 
//printf("%d %d  ",k,h);
switch (k)
	{
	case 1:
	{
	hh=cimag(*mymv); 
//	printf("%f %d ",hh,h);
	zh=0.0;
	zz=z;
	if (creal(z)<split)
		z=1-z;
	for (m=1;m<=myzt;m++)
		{
		hold=pow(-1.0,m-1)*cpow(m,-z);
		zh=zh+hold;
		mymod=cabs(hold);		
		if (!mft)
			if (mymod/cabs(zh)<tesc) //changed to be relative to value of function
				break;
		}
	zh=(1+cpow(5,0.5-z))/(1-cpow(2,1-z))*zh;		
	if(creal(zz)<split)
				zh=cpow(5,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*csin(zz*pi/2)*zh;
	z1=zh;
	zh=0.0;
	//zz=z;
	z=zz;
	if (creal(z)<split)
		z=1-z;
	n=0;
	v=1;
	for (m=1;m<=myzt;m++)
		{
		v=v55[n];
		hold=v*cpow(m,-z);
		zh=zh+hold;
		mymod=cabs(hold);		
		if (!mft)
			if ((v!=0)&&(mymod/cabs(zh)<tesc)) //changed to be relative to value of function
				break;
		n=n+1;
		if (n==5)
			n=0;
		}
	if(creal(zz)<split)
				zh=cpow(5,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*csin(zz*pi/2)*zh;
	z2=zh;
	z=z1*(1-hh/thou)+z2*hh/thou;
	if (myxi==0)
		return z;
	else
		return cpow(pi/5,-zz/2)*Lgamma((zz)/2)*z;	
	}
	
	case 9:
	{
	
	hh=cimag(*mymv);
//	printf("%f ",th); 
	th=th*hh/100.0;	
	tn=tan(th);
	chars(5,4);
	zh=cexp(-th*I)*zeta(z);
	chars(5,2);
	zh=zh+cexp(th*I)*zeta(z);
	//printf("*");
	(*mymv)=k+I*hh;
	return zh*sqrt(1+tn*tn)/2;
/*	hh=cimag(*mymv); 
//	printf("%f %d ",hh,h);
	zh=0.0;
	zz=z;
	if (creal(z)<split)
		z=1-z;
	n=0;
	w=1;
	for (m=1;m<=myzt;m++)
		{
		w=v54[n];
		if (creal(zz)<split)
			w=conj(w);
		hold=w*cpow(m,-z);
		zh=zh+hold;
		mymod=cabs(hold);		
		if (!mft)
			if ((w!=0)&&(mymod/cabs(zh)<tesc)) //changed to be relative to value of function
				break;
		n=n+1;
		if (n==5)
			n=0;
		}
	if(creal(zz)<split)
			//	zh=(1.175571+I*1.902113)/5*cpow(5/2/pi,zz)*Lgamma(z)*ccos((zz)*pi/2)*zh;
				zh=(1.175571-I*1.902113)/sqrt(5)*cpow(5,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*ccos((zz)*pi/2)*zh;				
	z1=zh;
	zh=0.0;
	//zz=z;
	z=zz;
	if (creal(z)<split)
		z=1-z;
	n=0;
	v=1;
	for (m=1;m<=myzt;m++)
		{
		w=v53[n];
		if (creal(zz)<split)
			w=conj(w);
		hold=w*cpow(m,-z);
		zh=zh+hold;
		mymod=cabs(hold);		
		if (!mft)
			if ((w!=0)&&(mymod/cabs(zh)<tesc)) //changed to be relative to value of function
				break;
		n=n+1;
		if (n==5)
			n=0;
		}
	if(creal(zz)<split)
				zh=(1.175571+I*1.902113)/sqrt(5)*cpow(5,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*ccos((zz)*pi/2)*zh;				
	z2=zh;
	th=th*hh/thou;
	tn=tan(th);
//	printf("%f ",th);
//	return sqrt(1.0+tn*tn)/2.0*(z1*cexp(-I*th)+z2*cexp(I*th));
//	return sqrt(1.0+tn*tn)/2*(z2+z1);
	return z1;*/
	}

default:
{
zh=0.0;
zz=z;
if (creal(z)<split)
	z=1-z;
n=0;
v=1;
for (m=1;m<=myzt;m++)
		{
		switch (k)
		{
		case 2:
			{ 
			hold=pow(-1.0,m-1)*cpow(m,-z);
			break;
			}
		case 3:
			{
			if (h==3)
				{
				hold=v3[n]*cpow(m,-z);
				v=v3[n];
				}			
			else
				hold=pow(-1.0,m-1)*cpow(m,-z);
			break;
			}
		case 4:
			{
			if (h==2)
				{
				hold=v4[n]*cpow(m,-z);			
				v=v4[n];
				}			
			else
				hold=pow(-1.0,m-1)*cpow(m,-z);
			break;
			}
		case 5:
			{
			if (h==1)
				{
				hold=v51[n]*cpow(m,-z);
				v=v51[n];
				break;
				}
			if (h==2)
				{
				hold=v52[n]*cpow(m,-z);
				v=v52[n];
				break;
				}
			if ((h==3)||(h==4))
				{
				hold=pow(-1.0,m-1)*cpow(m,-z);
				break;
				}
			if (h==5)
				{
				hold=v55[n]*cpow(m,-z);
				v=v55[n];
				break;
				}											
			}
		case 6:
			{
			hold=v6[n]*cpow(m,-z);			
			//hold=pow(-1.0,m-1)*cpow(m,-z);
			v=v6[n];			
			break;
			}
		case 7:
			{ 
			hold=v7[n]*cpow(m,-z);
			v=v7[n];			
			break;
			}
		case 8:
			{
			hold=v8[n]*cpow(m,-z);			
			v=v8[n];			
			break;
			}
		}
		//printf("%d %d %f ",m,n,v[n]);
		zh=zh+hold;
		mymod=cabs(hold);		
		if (!mft)
			if ((v!=0)&&(mymod/cabs(zh)<tesc)) //changed to be relative to value of function
				break;
		n=n+1;
		if (n==k)
			n=0;
		}
	switch (k)
	{
	case 2:
		{
		if (h % 2 ==0)
			zh=(1+cpow(2,0.5-z))/(1-cpow(2,1-z))*zh;
		else
			zh=(1-cpow(2,0.5-z))/(1-cpow(2,1-z))*zh;		
		break;
		}
	case 3:
		{
		if (h==2)
			zh=(1+cpow(3,0.5-z))/(1-cpow(2,1-z))*zh;
		if (h==1)
			zh=(1-cpow(3,0.5-z))/(1-cpow(2,1-z))*zh;		
		break;
		}
	case 4:
		{
		if (h==1)
			zh=(1-cpow(4,0.5-z))/(1-cpow(2,1-z))*zh;		
		break;
		}
	case 5:
		{
		if (h==3)
			zh=(1-cpow(5,0.5-z))/(1-cpow(2,1-z))*zh;		
		if (h==4)
			zh=(1+cpow(5,0.5-z))/(1-cpow(2,1-z))*zh;		
		break;
		}
	case 6:
		{
		if (h==1)
			zh=(1-(1-sqrt(2))/(1+cpow(2,z)))*zh;
		else
			zh=(1-(1+sqrt(2))/(1+cpow(2,z)))*zh;
		break;
		}
	case 8:
		{
		if (h==1)
			zh=(1-cpow(2,0.5-z))*zh;		
		break;
		}
	}
if(creal(zz)<split)
		if ((h % 2 ==0)||(k==7)||(k==8)||((k==5)&&(h==5)))
			if ((k==5)&&(h==2)||(k==7)||(k==8)||(k==6))
				zh=cpow(k,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*csin((zz-1)*pi/2)*zh; //-ccos
			else					
				zh=cpow(k,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*csin(zz*pi/2)*zh;
		else
			if (((k==5)&&(h==1))||(k==6)||((k==3)&&(h==3)))
				zh=cpow(k,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*ccos((zz)*pi/2)*zh;			
			else					
				zh=cpow(k,0.5-zz)*2*cpow(2*pi,-z)*Lgamma(z)*ccos((zz+1)*pi/2)*zh; //-csin	
if (myxi==0)
	return zh;
else
	{
		if ((h % 2 ==0)||(k==7)||(k==8)||((k==5)&&(h==5)))
			if ((k==5)&&(h==2)||(k==7)||(k==8)||(k==6))
				//return pow(-1,(creal(zz)<split))*cpow(pi/k,-zz/2)*Lgamma((1+zz)/2)*zh;			
				return cpow(pi/k,-zz/2)*Lgamma((1+zz)/2)*zh;			
			else					
				return cpow(pi/k,-zz/2)*Lgamma((zz)/2)*zh;
		else
			if (((k==5)&&(h==1))||(k==6)||((k==3)&&(h==3)))
				return cpow(pi/k,-zz/2)*Lgamma((1+zz)/2)*zh;			
			else					
				//return pow(-1,(creal(zz)<split))*cpow(pi/k,-zz/2)*Lgamma((1+zz)/2)*zh;	
				return cpow(pi/k,-zz/2)*Lgamma((zz)/2)*zh;	
	}
}
}
}

double _Complex powerseries(register double _Complex z)
{
double _Complex zh,zz;
int m,k,h,j;
double  mymod,n;
k=(int)(creal(*mymv)); 
h=(int)(cimag(*mymv));
zz=0.0;
m=1;
n=1;
if ((cabs(z)>1)&&(esco)) return 0.0001*I;
for (j=1;j<=myzt;j++)
		{
//		 printf("%d %f  ",m,creal(*(mymv+m)));
		 if (*(mymv+m)!=0)
			{
			zh=(*(mymv+m))*cpow(z,j);///n;
			if(h==-999)
				zh=(*(mymv+m))*cpow(z,*(mymv+k+m)+j);///n;
			if((h==-1)&&(k==1))
				zh=pow(-1,j-1)*cpow(z,j);///n;				
		    if (isnan(zh))
			break;			
			zz=zz+zh;
		
			mymod=cabs(zh);
			if (!mft)
				if (mymod/cabs(zz)<tesc) //changed to be relative to value of function
					break;
			}
			m=m+1;
			if(m==k+1)
				m=1;
			n=n*(j);
			
		}
if (orbto) zz=zz+1;
return zz;
}

double _Complex fourier(register double _Complex z)
{
double _Complex zh,zz;
int m,k,h,j;
double  mymod,n;
k=(int)(creal(*mymv)); 
h=(int)(cimag(*mymv));
zz=0.0;
m=1;
//n=1;
if ((cimag(z)<0)&&(esco)) return 0.0001*I;
for (j=1;j<=myzt;j++)
		{
//		 printf("%d %f  ",m,creal(*(mymv+m)));
		 if (*(mymv+m)!=0)
			{
			//zh=(*(mymv+m))*cpow(z,j)/n;
			zh=(*(mymv+m))*cexp(2*pi*I*z*j);
			if(h==-999)
				zh=(*(mymv+m))*cexp(2*pi*I*z*(*(mymv+k+m)+j));
			if((h==-1)&&(k==1))
				zh=pow(-1,j-1)*cexp(2*pi*I*z*j);				
		    if (isnan(zh))
			break;			
			zz=zz+zh;
		
			mymod=cabs(zh);
			if (!mft)
				if (mymod/cabs(zz)<tesc) //changed to be relative to value of function
					break;
			}
			m=m+1;
			if(m==k+1)
				m=1;
			//n=n*(j);
			
		}
if (orbto) zz=zz+1;
return zz;
}

double _Complex titch(register double _Complex z)
{
double _Complex zh,hold;
int lv,m,k,prd;
double si=(sqrt(10.0-2.0*sqrt(5.0))-2.0)/(sqrt(5.0)-1.0);
double th=atan2(1,si);
double tn,h;
k=(int)(creal(*mymv)); 
h=cimag(*mymv);
h=1000;
th=th*h/1000;
//printf("%f ",h);
tn=tan(th);
chars(5,4);
zh=cexp(-th*I)*zeta(z);
chars(5,2);
zh=zh+cexp(th*I)*zeta(z);
//zh=zeta(z,1);
return zh*sqrt(1+tn*tn)/2;
}


double _Complex hurw(register double _Complex z)
{
double _Complex zh,hold;
int lv,m,k,h,prd,hh;
k=(int)(creal(*mymv)); 
h=(int)(cimag(*mymv));
//hh=h;
//if (h>k)
//	h=h % k;
//printf("%d %f  ",h,cimag(*mymv)); 
//printf("%d %d  ",k,hh);
if (k==1)
	{
	z=zetaa(z);
	return z;
	}
if((k==2)&&(h==1))
	{
	z=(cpow(2,z)-1)*zetaa(z);
	return z;
	}
lv=chars(k,2);
//printf("%d %d  ",lv,(int)(creal(*mymv)));
zh=0;
for(m=1;m<=lv;m++)
	{
	chars(k,m);
	prd=(int)(creal(*mymv));
	//printf("%d ",prd);

	hold=*(mymv+(h % prd));
//	hold=*(mymv+h);
	if ((prd==1)||(h==1))
		hold=1;
//if (m==4)
	zh=zh+conj(hold)*zeta(z);
	}
//if (hh>k)	
	zh=cpow(k,z)*zh;
	zh=zh/lv;
//printf("%d ",h);
*mymv=k+I*h;
return zh;
}

double _Complex bitz(register double _Complex z)
{
double x,y,pos,mw,mh;
if (mymw==0)
	return 1;
x=(int)(creal(z));
y=(int)(cimag(z));
mw=mymw/2;
mh=mymh/2;
if (mymw%2==0)
	{
	x=x-1.0/2;
	mw=mw-1.0/2;
	}
if (mymh%2==0)
	{
	y=y-1.0/2;
	mh=mh-1.0/2;
	}
pos=myml/2+x-y*mymw;//-1;
if ((abs(x)<=mw)&&(abs(y)<=mh))
		{
	//	printf("%f %f %f %f %f %f %d\n",creal(z),cimag(z),mw,mh,x,y,pos);		
		z=*(mymfl+(int)pos);
		}
	else
        z=0;//0.01*I;
return z;
}

double _Complex weier(register double _Complex z)
{
double _Complex zo,zz,zh,k,h;
int kk,hh,sq,m,n;
kk=(int)creal(*(mymv));
hh=(int)cimag(*(mymv));
k=I; 
h=1.0; 
if(kk>1)
	{
	k=*(mymv+1);
	h=*(mymv+2);
	}
zo=1.0/z/z;
sq=(int)(sqrt(myzt));
//sq=(int)myzt;
for (m=-sq;m<=sq;m++)
	for (n=-sq;n<=sq;n++)
		{
		//printf("%d %d\n",m,n);		
		zz=m*k+n*h;
		if((m!=0)||(n!=0))					
			zh=1.0/(z+zz)/(z+zz)-1.0/zz/zz;
		if(isnan(zh))
			zh=0;//1.0e+33;
		zo=zo+zh;
		}
return zo;
}

double _Complex moib(register double _Complex z,int m)
{
//double z1, z2, z3, a, b, c, d, k;
double a, b, c, d, k;

//z1=log(1+exp(-1.0/2))/log(2)-1.0/2; //.1839
//z2=log(1+exp(1.0/2))/log(2)-1.0/2; //.9053
//z3=-100.0/2;
k=40.0;
a=k+1.0/2.0;  //(z2-z3);
b=1.0-a;    //z1*a;
c=1.0;      //z2-z1;
d=1.0;      //-z3*c;
if (m)
	z=(a*z+b)/(c*z+d);
else
	z=(d*z-b)/(-c*z+a);
return z;
}

double _Complex myfn(register double _Complex z)
{
//int j;
//double _Complex zs;

switch (thisfn)
 {
case 1:
	 {z=zeta(z); break; }
case 2:
	{ 	z=ded(z); break; }
case 3:
	{ 	z=daven(z); break; }
case 4:
	{ 	z=zetar(z); break; }
case 5:
	{ 	z=hurw(z); break; }
case 6:
	{ 	z=1/zeta(z); break; }
case 7:
	{ 	z=zeta(z-1)*zeta(z); break; }
case 8:
	{ 	z=zeta(2*z)/zeta(z); break; }
case 9:
	{ 	z=-dzeta(z)/zeta(z); break; }
case 10:
	{ 	z=zeta(z-1)/zeta(z); break; }
case 11:
	{  z=dzeta(z); break; }
case 12:
	{ 	z=z-zeta(z)/dzeta(z); break; }
case 13:
	{ 	z=z+zeta(z)/dzeta(z); break; }
case 14:
	{ 	z=z-xi(z)/dxi(z); break; }
case 15:
	{ 	z=zetap(z); break; }
case 16:
	{ 	z=przeta(z); break; }
case 17:
	{ 	z=bitz(z); break; }
case 18:
	{ 	z=Lgamma(z); break; }
case 19:
	{ 	z=dgamma(z); break; }
case 20:
	{ 	z=digamma(z); break; }
case 21:
	{ 	z=theta(z); break; }
case 22:
	{ 	z=rsz(z); break; }
case 23:
	{ 	z=zeta(z)-z;	 break; }
case 24:
	{ 	z=z*zeta(z); break; }
case 25:
	{ 	z=cpow(pi,-z/2)*zeta(z); break; }
case 26:
	{ 	z=powerseries(z); break; }
case 27:
	{ 	z=fourier(z); break; }
case 28:
	{ 	z=cpow(myzt,-z); break; }
case 29:
	{ 	z=1/(1-cpow(myzt,-z)); break; }
case 30:
	{ 	z=(cpow(myzt,1-z))/(1-z); break; } //integral version of zeta series
case 31:
	{ 	z=z*cexp(-z); break; }
case 32:
	{ 	z=weier(z); break; }
	}
return z;
	}
	
int mandelLine(double _Complex k,int adc, int dz, int thisfni, int ft, int crval, int esconly, int fill, int mycol, int orbt, double myepsilon, double zterms, int Pgamma, int mandel, double originX, double originY, double escape_radius, double x,double y,double step,int maxdwell,int w,double * buf, long * myp, double _Complex * myv, double _Complex * myfl, int myw, int myh, int dol, int dom, int flip,int usxi)
{
//const double sfac = 0.02;
	register double Zr, Zi;
	register int dwell;
	register double _Complex z, c; 
	int i, i3, j, total, myloop, color, pflag, myflag, pd,orbf,stdw,dzsw,eflag;
	double _Complex orbit[1024+1],kh,h;
	double a,b,re,im,re2,im2,psx,psy;
	total = 0;
	//escape_radius=1e2;
	double tiny=0.001;
	//printf("%s\n",mystr);
	int myloopmax=18;
	if ((mycol==1)&&(orbt==1))
	{
		myloopmax=35;
		orbt=0;
	}	
	//printf ("max dwells: %d\n", maxdwell);
	const double pi2 = 2*3.1415926535;
	const double myfrac=1.0/2000;
	double ahalf=0;// image of a zero on 0.5 is 0!
	const int dwstk=0;
	const double stlk=0.006;
	//int myzti,thisfn;
	 
  mft=ft % 2;
  dzsw=(ft / 2) % 2;
  mmel=ft / 4;
  myxi=usxi;
  myzt=(long)zterms;
  altgam=Pgamma;
  mystep=step;
  thisfn=thisfni;
  escape_radius=pow(10,escape_radius);
  mymp=myp;
  mymv=myv;
  mymw=myw;
  mymh=myh; 
  myml=mymw*mymh;
  mymfl=myfl;
  esco=esconly;
  orbto=orbt;
/*  if (thisfn==18)
	for (i=0;i<=myzt-1;i++)
		printf("%d ",*(mymp+i));
  if (thisfn==18)
	{
	myp = calloc(myzt,sizeof(int)); 
	prims();
	}*/
  mdz=dz;
  if(mdz==-1)
	split=-100.0;
  if(mdz==0)
	split=0.0;
  if(mdz==1)
	split=0.25;
  if(mdz==2)
	split=0.5;
 // printf("%f ",split);
  //Zrr=999;
  tiny=pow(10,myepsilon)*step; //0.001;
//printf("%f %f\n", creal(ded(5)), cimag(ded(5)));
  if (mandel == 2)
	{
	if (k!=0)
		{
		kh=myfn(k);
		//printf("%f %f\n", creal(k), cimag(k));
		}
    for ( i = 0; i < w; x += step, i++ ) 
		{
		i3=3*i;
		if (flip)
			z=y-x*_Complex_I;
		else
			z=x+y*_Complex_I;
		c=0;
		if (dom)
			{
			z=moib(z,1);
		//	c=moib(c,1);
			}
		if (k!=0)
			{
			if (adc)
				z=z+kh;
			else
				{
				z=z*kh;
			//	printf("%f %f\n", creal(k), cimag(k));
				}
			}
		if (dzsw)
			{
			h=mystep*myfrac*(1+1*_Complex_I);
			z=(myfn(z+h)-myfn(z))/h;			
			}
		else
			z=myfn(z);
		if (k!=0)
			z=z-kh;
		if (dol)
			z=clog(z);
		if ((mycol!=1)&&(mycol!=3))
			{
			buf[i3]=pi;//-1;
			buf[i3+1]=cabs(z);
			buf[i3+2]=carg(z);			
			}
		else
			{
			buf[i3]=creal(z);
			buf[i3+1]=cimag(z);
			buf[i3+2]=carg(z);
			}
		}
	}
  else if (mandel == 1) //**MANDELBROT**
  {
   for ( i = 0; i < w; x += step, i++ ) {
		Zr=originX;
		Zi=originY;
		i3=3*i;
		color=1;
		myflag=0; // set period flag to 0
		eflag=0; // set escape flag to 0

		orbf=0;
		psx=stlk;
		psy=stlk;
		pd=0; // set period to 0 (no period)
		if(flip)
			c=y-x*_Complex_I;
		else
			c=x+y*_Complex_I;
		z=Zr+Zi*_Complex_I;
		if (crval)
			{
			if (adc)
				z=z+c;
			else
				z=c*z;
			   Zr=creal(z);
			   Zi=cimag(z);			
			}
		if (dom)
			   {
			   z=moib(z,1);
			   c=moib(c,1);
			   Zr=creal(z);
			   Zi=cimag(z);
			   }	
		for (dwell = 0; dwell < maxdwell; dwell++) {
			z=Zr+Zi*_Complex_I;
			z=myfn(z);
			if (dol)
				z=clog(z);
			if (adc)
				z=z+c;
			else
				z=c*z;
			orbit[dwell]=z;
			Zr=creal(z);
			Zi=cimag(z);	
            if (orbt)
            {
                if ((-Zr > escape_radius)||(isnan(Zr))) break;// point escapes to infinity
            }
            else
            {        if ((fabs(Zr) > escape_radius)||(isnan(Zr))) break;// point escapes to infinity
            }
           /* if ((orbt)&&(dwell>dwstk)&&((fabs(Zr)<stlk)||(fabs(Zi)<stlk)))  //orbit trap option
                {
                    orbf=1;
                    if (fabs(Zr-ahalf)<psx)
                    {
                        psx=fabs(Zr-ahalf);
                        stdw=dwell;
                    }
                    if (fabs(Zi)<psy)
                    {
                        psy=fabs(Zi);
                        stdw=dwell;	
                    }
                    
                    //break;  //imaginary axis orbit trap option
                }*/

			if (!esconly) {
				myloop=myloopmax; // periodicity loop
				if (dwell-1<myloop)
					myloop=dwell-1;
				re=creal(z);
				im=cimag(z);
				if (creal(z)>1)
				   color=3; // color blue if escapes to positive reals
				pflag=1; // flag to see if orbit remains positive real
				for (j=dwell-1; j>=dwell-myloop; j--)
				   {
				   re2=re-creal(orbit[j]);
				   im2=im-cimag(orbit[j]);
				   if (creal(orbit[j])<0)
					   pflag=0; // orbit went negative real
				   if (re2*re2+im2*im2<tiny) // we found a period!
					   {
					   if (pflag==1)
						   color=2; // if orbit never goes negative color green
					   myflag=1;
					   pd=dwell-j;
					   break; // break out of period test
					   }
				    }
				/*if (creal(z)>1)
				{
					eflag=1;
				}*/
				if (myflag==1)
				   break; // break out of dwell loop		   
				}
			}
		//	for (j=0; j<=2; j++)
		//		buf[i3+j]=-1;
			if ((dwell == maxdwell)||(eflag==1))
				buf[i3] = -1;
			else
				{
				if (crval)
					dwell++;
				if (Zr>escape_radius)
					if (mycol==3)
						buf[i3]=0.003*log(creal(z))/Lgamma(dwell+1)*pow(pi2,dwell+1); //potential method mycol 3
					else
						buf[i3] = dwell;
				else if (-Zr>escape_radius)
					if (mycol==3)
						buf[i3]=0.003*log(creal(-z))/Lgamma(dwell+1)*pow(pi2,dwell+1); //potential method mycol 3
					else
						buf[i3] = dwell+1;
	//			else if ((orbt)&&(fabs(Zr)<0.01))
	//				buf[i3]=dwell+20;
				else if ((orbt)&&(orbf))
					buf[i3] = stdw+15-2*log(psx)-3*log(psy);
				else
					buf[i3] = -(dwell+2);
					buf[i3+1]=pd;
					//for (j=0; j<=2; j++)
						//buf[i3+j]=dwell;
				}
			total += dwell;
			}
		//printf("*");

		}
	else if (mandel== 0) //**JULIA**//
	{
	a=-0.2959050055752;
	b=25.01085758;
	c=originX+originY*_Complex_I;
	//c=0; //testing testing ....
	for ( i = 0; i < w; x += step, i++ ) 
		{
		i3=3*i;
		color=1;
		if((thisfn==14)&&(mycol==1))
			color=2;
		myflag=0; // set start color to red
		orbf=0;
		psx=stlk;
		psy=stlk;
		pd=0; // set period to 0 (no period)
		if(flip)
			{Zr = y; Zi = -x;}
		else
			{Zr = x; Zi = y;}
		z=Zr+Zi*_Complex_I;
		c=originX+originY*_Complex_I;
		if (dom)
			   {
		//	   c=0.9753;
			   z=moib(z,1);
			   c=moib(c,1);
		//	   printf("%f\n",creal(c));
			   Zr=creal(z);
			   Zi=cimag(z);
			   }	
		for (dwell = 1; dwell < maxdwell; dwell++) // dwell (iteration) loop
			{
			z=Zr+Zi*_Complex_I;
		    z=myfn(z);
			if (dol)
				z=clog(z);
			if (adc)
				z=z+c;
			else
				z=c*z;
			orbit[dwell]=z;
			Zr=creal(z);
			Zi=cimag(z);	
            if (orbt)
                {
                    if ((-Zr > escape_radius)||(isnan(Zr))) break;// point escapes to infinity
                }
                else
                {        if ((fabs(Zr) > escape_radius)||(isnan(Zr))) break;// point escapes to infinity
                }
               /* if ((orbt)&&(dwell>dwstk)&&((fabs(Zr)<stlk)||(fabs(Zi)<stlk)))  //orbit trap option
                    {
                        orbf=1;
                        if (fabs(Zr-ahalf)<psx)
                        {
                            psx=fabs(Zr-ahalf);
                            stdw=dwell;
                        }
                        if (fabs(Zi)<psy)
                        {
                            psy=fabs(Zi);
                            stdw=dwell;
                        }
                        
                        //break;  //imaginary axis orbit trap option
                    }*/

			if (!esconly) {
				myloop=myloopmax; // periodicity loop
				if (dwell-1<myloop)
					myloop=dwell-1;
				re=creal(z);
				im=cimag(z);
				if (creal(z)>10)
				   color=2; // color blue if escapes to positive reals
				pflag=1; // flag to see if orbit remains positive real
				for (j=dwell-1; j>=dwell-myloop; j--)
				  {
				   re2=re-creal(orbit[j]);
				   im2=im-cimag(orbit[j]);
				   if (creal(orbit[j])<0)
					   pflag=0; // orbit went negative real
				   if (re2*re2+im2*im2<tiny) // we found a period!
					   {
					   if (pflag==1)
						   color=2; // if orbit never goes negative color green
					   myflag=1;
					   pd=dwell-j;
					   break; // break out of period test
					   }
					}
				if (myflag==1)
				   break; // break out of dwell loop		   
				}
			}
		//if (thisfn==14)
		//	if ((cabs(z-(0.5+b*_Complex_I))<0.01)||(cabs(z-(0.5-b*_Complex_I))<0.01))
		//		color=1;
		if (myflag==0)
			color=0;
		if ((fabs(Zr) > escape_radius)||(isnan(Zr))) // point escapes to infinity
			color=3;
		if ((mycol!=3)&&(creal(-z)>escape_radius))
			dwell=dwell+1;
		else
			dwell=dwell;
		if((mycol==0)||((mycol>=2)&&(mycol<5)))
			{
			if (dwell == maxdwell)
				buf[i3] = -1;
		//	else if ((orbt)&&(fabs(Zr)<0.01))
		//		buf[i3]=dwell+20;
		else if ((orbt)&&(orbf))
				buf[i3] = stdw+15-2*log(psx)-3*log(psy);
			else
				{
				if ((mycol==2)&&(color==2))
					dwell=dwell+256;
				if ((mycol==2)&&(color==1))
					dwell=dwell+16;
				buf[i3] = dwell;
				if (mycol==3)
					if ((fabs(Zr) > escape_radius)||(fpclassify(creal(z))== FP_NAN))
							buf[i3]=0.003*log(fabs(creal(z)))/Lgamma(dwell+1)*pow(pi2,dwell+1); //potential method mycol 3
				}
			}
		if (mycol==5)
			{
			for (j=0; j<=2; j++)
				buf[i3+j]=-1;
			if (color==0)
				for (j=0; j<=2; j++)
					buf[i3+j]=dwell;
			else
				{
				buf[i3+color-1]=dwell;
				if (pd>0)
					{
					buf[i3+2]=-pd/myloopmax-1;
					}
				}
			}
		 if (mycol==1)
		 {
			 if (dwell == maxdwell)
				 buf[i3] = -1;
			 else
			 {
				 for (j=0; j<=2; j++)
					 buf[i3+j]=dwell;
				 if (pd>0)
				 {
					 buf[i3] = -(dwell+2);
					 buf[i3+1]=pd;
					 //buf[i3+2]=rt;
				 }
			 }
		 }
			//buf[i3]=dwell;
		total += dwell;

	} }
	return total;
	/*if (thisfn==18)
		free(myp);*/
}