// compute one scan line of mandelbrot set

#include "mandel.h"
#include <complex.h>
#include <math.h>

double _Complex myaltgamma(double _Complex z, int myzt)
{
int n;
double _Complex f;

f=1/z;
for (n=1; n<=myzt; n++ ) 
    f=f*cpow(1.0+1.0/n,z)/(1.0+z/n);
return f;
}


double _Complex mygamma(double _Complex z)
//complex gamma function using Lanczos approximation to extend to the entire plane. 
{
	double _Complex zz, t, LogofGamma, s, ss, f;

	double pi=3.14159;
	double twopi;
	double c[12];
	int k,g;

	c[1] = 1.000000000000000174663;
	c[2] = 5716.400188274341379136;
	c[3] = -14815.30426768413909044;
	c[4] = 14291.49277657478554025;
	c[5] = -6348.160217641458813289;
	c[6] = 1301.608286058321874105;
	c[7] = -108.1767053514369634679;
	c[8] = 2.605696505611755827729;
	c[9] = -0.7423452510201416151527e-2;
	c[10] = 0.5384136432509564062961e-7;
	c[11] = -0.4023533141268236372067e-8;
	twopi=pi+pi;
	g=9;
	s=0;
	zz=z;
	if ((round(creal(zz))==creal(zz))&&(cimag(zz)==0)&&(creal(zz)<=0))
		{
		f=10e308;
		}
	else
	{
		if (creal(z)<0)
			z=-z;
		t=z+g;
		for ( k = g+2; k>=2; k-- ) 
			{
			s=s+c[k]/t;
			t=t-1;
			//printf("%d\n",k);
			}
		s=s+c[1];
		ss=(z+g-0.5);
		s=clog(s*sqrt(twopi))+(z-0.5)*clog(ss)-ss;
		LogofGamma = s;
		f = cexp(LogofGamma);
		if (creal(zz)<0)
			{
			f=-pi/(z*f*csin(pi*z));
			}
		
	}
	return f;
}
    

double _Complex myzeta(double _Complex z, int myzt, int altgam)
	{
		double pi=3.14159;
		double _Complex zz, hold, f;
		int steps, sw, i;

		zz=z;
		if (creal(z)<0)
			z=1-z;
		//steps=2002;
		steps=myzt;
		f=0;
		sw=-1;
		for (i = 1; i<=steps; i++ ) 
		{
			sw=-sw;
			hold=sw/cpow(i,z);
			f=f+hold;
			if (cabs(hold)<.001)
				break;
		}
	sw=-sw;
	f=f+sw/cpow((i+1),z)/2;
	f=f/(1-cpow(2,(1-z)));
	if(creal(zz)<0)
		{
		if (altgam)
			f=2*cpow(2*pi,-z)*ccos(z*pi/2)*myaltgamma(z,myzt)*f;
		else
			f=2*cpow(2*pi,-z)*ccos(z*pi/2)*mygamma(z)*f;
		}
	return f;
	}

double _Complex myfn(double _Complex z, double _Complex c, int myzt, int altgam)
{
//double _Complex f;
//z=c*z*cexp(z);
z=myzeta(z,myzt,altgam)+c;
//z=mygamma(z)+c;
return z;
}	
	
int mandelLine(int fill, int mycol,  int myzt, int altgam, int mandel, double originX, double originY, double escape_radius, double x,double y,double step,int maxdwell,int w,double * buf)
{
//const double sfac = 0.02;
	register double Zr, Zi;
	register int dwell;
	int i, i3, j, total, myloop, color, pflag, myflag, pd;
	double _Complex z, c, orbit[1024+1];
	double a,re,im,re2,im2;
	total = 0;
	//escape_radius=1e2;
	double tiny=0.001;
	//printf("ER=%f\n",escape_radius);
	int myloopmax=20;
	
  if (mandel == 2)
	{
    for ( i = 0; i < w; x += step, i++ ) 
		{
		i3=3*i;
		z=x+y*_Complex_I;
		c=0;
		//z=myfn(z,c,myzt);
		z=myzeta(z,myzt,altgam)+c;
		if (mycol)
			{
			buf[i3]=-1;
			buf[i3+1]=cabs(z);
			buf[i3+2]=carg(z)+3.14159;			
			}
		else
			{
			buf[i3]=creal(z);
			buf[i3+1]=cimag(z);
			buf[i3+2]=carg(z)+3.14159;
			}
		}
	}
  else if (mandel == 1) {
   for ( i = 0; i < w; x += step, i++ ) {
		//Zr=999.0;
		Zr=originX;
		Zi=originY;
		//Zr=1.465; //gamma
		//Zi=0;
		i3=3*i;
		color=1;
		myflag=0; // set start color to red
		pd=0; // set period to 0 (no period)
		c=x+y*_Complex_I;
		for (dwell = 0; dwell < maxdwell; dwell++) {
		z=Zr+Zi*_Complex_I;
		//z=myfn(z,c,myzt);
		z=myzeta(z,myzt,altgam)+c;
		orbit[dwell]=z;
		Zr=creal(z);
		Zi=cimag(z);	
			//if((fabs(Zr) > escape_radius)||((dwell>3)&&(fabs(Zr)<0.01))) break;  //orbit trap option
			if(fabs(Zr) > escape_radius) break;
			myloop=myloopmax; // periodicity loop
			if (dwell-1<myloop)
				myloop=dwell-1;
		    re=creal(z);
		    im=cimag(z);
		    if (creal(z)>10)
			   color=2; // color blue if escapes to positive reals
			pflag=1; // flag to see if orbit remains positive real
			for (j=dwell-1; j>=dwell-myloop; j--)
			  {
			   re2=re-creal(orbit[j]);
		       im2=im-cimag(orbit[j]);
			   if (creal(orbit[j])<0)
				   pflag=0; // orbit went negative real
			   if (re2*re2+im2*im2<tiny) // we found a period!
				   {
				   if (pflag==1)
					   color=2; // if orbi never goes negative color green
				   myflag=1;
				   pd=dwell-j;
				   break; // break out of period test
				   }
			    }
			if (myflag==1)
			   break; // break out of dwell loop		   

		}
	//	for (j=0; j<=2; j++)
	//		buf[i3+j]=-1;
		if (dwell == maxdwell)
			buf[i3] = -1;
		else
			if (Zr>escape_radius)
				buf[i3] = dwell;
			else if (-Zr>escape_radius)
				buf[i3]=dwell+1;
			//else if (fabs(Zr)<0.01)
				//buf[i3]=dwell+20;
			else
				buf[i3] = -(dwell+2);
				buf[i3+1]=pd;
				//for (j=0; j<=2; j++)
					//buf[i3+j]=dwell;
		total += dwell;
		}
		//printf("*");
		}
	else if (mandel== 0)
	{
	a=-0.2959050055752;
	c=originX+originY*_Complex_I;
	//c=0; //testing testing ....
	for ( i = 0; i < w; x += step, i++ ) 
		{
		i3=3*i;
		color=1;
		myflag=0; // set start color to red
		pd=0; // set period to 0 (no period)
		Zr = x; Zi = y;
		for (dwell = 1; dwell < maxdwell; dwell++) // dwell (iteration) loop
			{
			z=Zr+Zi*_Complex_I;
			//z=myfn(z,c,myzt);
			z=myzeta(z,myzt,altgam)+c;
			orbit[dwell]=z;
			Zr=creal(z);
			Zi=cimag(z);	
			if ((fabs(Zr) > escape_radius)||(fpclassify(creal(z))== FP_NAN)) // point escapes to infinity
				break;				
			myloop=myloopmax; // periodicity loop
			if (dwell-1<myloop)
				myloop=dwell-1;
		    re=creal(z);
		    im=cimag(z);
		    if (creal(z)>10)
			   color=2; // color blue if escapes to positive reals
			pflag=1; // flag to see if orbit remains positive real
			for (j=dwell-1; j>=dwell-myloop; j--)
			  {
			   re2=re-creal(orbit[j]);
		       im2=im-cimag(orbit[j]);
			   if (creal(orbit[j])<0)
				   pflag=0; // orbit went negative real
			   if (re2*re2+im2*im2<tiny) // we found a period!
				   {
				   if (pflag==1)
					   color=2; // if orbi never goes negative color green
				   myflag=1;
				   pd=dwell-j;
				   break; // break out of period test
				   }
			    }
			if (myflag==1)
			   break; // break out of dwell loop		   
		    }
		if (myflag==0)
			color=0;
		if ((fabs(Zr) > escape_radius)||(fpclassify(creal(z))== FP_NAN)) // point escapes to infinity
			color=3;
		if (creal(z)>escape_radius)//||isnan(z)	
			dwell=dwell+1;
		else
			dwell=dwell;
		if ((mycol==0)||(mycol==2))
			{
			if (dwell == maxdwell)
				buf[i3] = -1;
			else
				buf[i3] = dwell;
			}
		else
			{
			for (j=0; j<=2; j++)
				buf[i3+j]=-1;
			if (color==0)
				for (j=0; j<=2; j++)
					buf[i3+j]=dwell;
			else
				{
				buf[i3+color-1]=dwell;
				if (pd>0)
					buf[i3+2]=pd;
				}
			}
			//buf[i3]=dwell;
		total += dwell;
	} }
	return total;
}

